package com.hz.pm.api.common.helper;

import lombok.RequiredArgsConstructor;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.Process;
import org.flowable.bpmn.model.UserTask;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.TaskService;
import org.flowable.engine.history.HistoricActivityInstance;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.task.api.Task;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.flowable.variable.api.history.HistoricVariableInstance;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * WflowHelper
 * </p>
 *
 * @author WendyYang
 * @since 16:00 2024/3/11
 */
@Component
@RequiredArgsConstructor
public class WflowHelper {

    private final HistoryService historyService;
    private final RepositoryService repositoryService;
    private final TaskService taskService;

    public boolean isOrUserTask(String procDefId, String activityId) {
        List<Process> processes = repositoryService.getBpmnModel(procDefId)
                .getProcesses();
        if (processes.isEmpty()) {
            return false;
        }
        List<UserTask> userTasks = processes.get(0).findFlowElementsOfType(UserTask.class);
        for (UserTask userTask : userTasks) {
            if (userTask.getId().equals(activityId)) {
                return !userTask.getLoopCharacteristics().isSequential();
            }
        }
        return false;
    }

    public List<Task> listPendingTasks(String activityId) {
        return taskService.createTaskQuery()
                .taskDefinitionKey(activityId)
                .list();
    }

    public List<HistoricTaskInstance> listFinishedTasks(String activityId) {
        return historyService.createHistoricTaskInstanceQuery()
                .taskDefinitionKey(activityId)
                .list();
    }

    /**
     * 所有流程是否完成（审核通过）
     *
     * @param instCodes \
     * @return \
     */
    public boolean allInstancesFinished(Set<String> instCodes) {
        List<HistoricProcessInstance> finishedInstances = historyService.createHistoricProcessInstanceQuery()
                .processInstanceIds(instCodes)
                .finished()
                .list();
        if (finishedInstances.size() != instCodes.size()) {
            return Boolean.FALSE;
        }
        for (HistoricProcessInstance instance : finishedInstances) {
            String processDefinitionId = instance.getProcessDefinitionId();
            UserTask userTaskDefineNode = getLastUserTaskDefineNode(processDefinitionId);
            List<HistoricActivityInstance> finishedTasks = historyService.createHistoricActivityInstanceQuery()
                    .activityType("userTask")
                    .activityId(userTaskDefineNode.getId())
                    .processInstanceId(instance.getId())
                    .orderByHistoricActivityInstanceEndTime()
                    .desc()
                    .list();
            if (finishedTasks.isEmpty()) {
                return Boolean.FALSE;
            }
            List<HistoricVariableInstance> variableInstances = finishedTasks.stream()
                    .map(w -> {
                        String variableName = "approve_" + w.getTaskId();
                        return historyService.createHistoricVariableInstanceQuery()
                                .processInstanceId(instance.getId())
                                .variableName(variableName)
                                .singleResult();
                    }).filter(w -> w != null && "PASS".equals(w.getValue().toString()))
                    .collect(Collectors.toList());
            if (!userTaskDefineNode.getLoopCharacteristics().isSequential()) {
                if (variableInstances.isEmpty()) {
                    return Boolean.FALSE;
                }
            } else {
                if (variableInstances.size() != finishedTasks.size()) {
                    return Boolean.FALSE;
                }
            }
        }
        return Boolean.TRUE;
    }

    /**
     * 获取流程定义的最后一个用户任务节点
     *
     * @param processDefinitionId processDefinitionId
     * @return \
     */
    public UserTask getLastUserTaskDefineNode(String processDefinitionId) {
        BpmnModel bpmnModel = repositoryService.getBpmnModel(processDefinitionId);
        List<Process> processes = bpmnModel.getProcesses();
        if (!processes.isEmpty()) {
            List<UserTask> userTasks = processes.get(0).findFlowElementsOfType(UserTask.class);
            if (!userTasks.isEmpty()) {
                return userTasks.get(userTasks.size() - 1);
            }
        }
        return null;
    }

}
