package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.SelfTestStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderSelfTestStatus;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * <p>
 * 标段系统自测状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@EnableStateMachine(name = SelfTestStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class SelfTestStateMachineBuilderImpl implements BaseStateMachineBuilder<Purchase, SelfTestStateChangeEvent, TenderSelfTestStatus> {

    public static final String MACHINE_ID = "selfTestStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<TenderSelfTestStatus, SelfTestStateChangeEvent> build() throws StateMachineException {
        StateMachine<TenderSelfTestStatus, SelfTestStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：" + stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<TenderSelfTestStatus, SelfTestStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<TenderSelfTestStatus, SelfTestStateChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<TenderSelfTestStatus, SelfTestStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        builder.configureStates()
                .withStates()
                .initial(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO)
                .states(EnumSet.allOf(TenderSelfTestStatus.class));

        builder.configureTransitions()
                .withExternal()
                .source(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO)
                .target(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT)
                .event(SelfTestStateChangeEvent.SUBMIT_SELF_TEST_INFO)
                .and()
                .withExternal()
                .source(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT)
                .target(TenderSelfTestStatus.SELF_TEST_INFO_PASSED)
                .event(SelfTestStateChangeEvent.SELF_TEST_PASSED)
                .and()
                .withExternal()
                .source(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT)
                .target(TenderSelfTestStatus.SELF_TEST_INFO_FAILED)
                .event(SelfTestStateChangeEvent.SELF_TEST_FAILED)
                .and()
                .withExternal()
                .source(TenderSelfTestStatus.SELF_TEST_INFO_FAILED)
                .target(TenderSelfTestStatus.SELF_TEST_INFO_AUDIT)
                .event(SelfTestStateChangeEvent.RESUBMIT_SELF_TEST)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "selfTestStatePersister")
    public StateMachinePersister<TenderSelfTestStatus, SelfTestStateChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<TenderSelfTestStatus, SelfTestStateChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<TenderSelfTestStatus, SelfTestStateChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{} ==> {}", contextObj.getProjectId(), contextObj.getId());
            }

            @Override
            public StateMachineContext<TenderSelfTestStatus, SelfTestStateChangeEvent> read(Purchase contextObj) {
                TenderSelfTestStatus status = TenderSelfTestStatus.getNoNull(contextObj.getSelfTestStatus());
                return new DefaultStateMachineContext<>(status, null, null, null, null, MACHINE_ID);
            }
        });
    }

}
