package com.hz.pm.api.common.statemachine.builder.impl;

import cn.hutool.core.collection.ListUtil;
import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.statemachine.factory.TenderGuardFactory.PurchaseContractRecordToAcceptGuard;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.ITenderStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderAdaptStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderMainStatus;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.List;

import static com.hz.pm.api.common.statemachine.factory.TenderGuardFactory.PurchaseContractRecordToOperationGuard;

/**
 * <p>
 * 标段状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@RequiredArgsConstructor
@EnableStateMachine(name = TenderStateMachineBuilderImpl.MACHINE_ID)
public class TenderStateMachineBuilderImpl implements BaseStateMachineBuilder<Purchase, TenderStateChangeEvent, ITenderStatus> {

    public static final String MACHINE_ID = "tenderStateMachine";

    private final BeanFactory beanFactory;
    private final PurchaseContractRecordToAcceptGuard purchaseContractRecordToAcceptGuard;
    private final PurchaseContractRecordToOperationGuard purchaseContractRecordToOperationGuard;


    @Override
    public StateMachine<ITenderStatus, TenderStateChangeEvent> build() throws StateMachineException {
        StateMachine<ITenderStatus, TenderStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：{}", stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<ITenderStatus, TenderStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<ITenderStatus, TenderStateChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<ITenderStatus, TenderStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        List<ITenderStatus> states = ListUtil.toList(TenderMainStatus.values());
        states.add(TenderAdaptStatus.WITHOUT_ADAPT_INFO);

        builder.configureStates()
                .withStates()
                .initial(TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .states(new HashSet<>(states));
        builder.configureTransitions()
                // 填写采购&合同信息
                .withExternal()
                .source(TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .target(TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN)
                .event(TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .guard(purchaseContractRecordToOperationGuard)
                .and()
                .withExternal()
                .source(TenderMainStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .target(TenderMainStatus.FINALLY_INSPECTED_PASSED)
                .event(TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO)
                .guard(purchaseContractRecordToAcceptGuard)
                .and()
                // 填写实施计划
                .withExternal()
                .source(TenderMainStatus.TO_BE_SUBMIT_OPERATION_PLAN)
                .target(TenderAdaptStatus.WITHOUT_ADAPT_INFO)
                .event(TenderStateChangeEvent.SUBMIT_OPERATION_PLAN)
                .and()
                .withExternal()
                .source(TenderMainStatus.WAIT_ORG_CONFIRM)
                .target(TenderMainStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO)
                .event(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM)
                .and()
                // 上传初验材料
                .withExternal()
                .source(TenderMainStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO)
                .target(TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY)
                .event(TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES)
                .and()
                // 终验申报
                .withExternal()
                .source(TenderMainStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY)
                .target(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY)
                .event(TenderStateChangeEvent.SUBMIT_FINALLY_INSPECTED)
                .and()
                // 终验申报重新提交
                .withExternal()
                .source(TenderMainStatus.FINALLY_INSPECTED_FAILED)
                .target(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY)
                .event(TenderStateChangeEvent.RESUBMIT_FINALLY_INSPECTED)
                .and()
                // 终验申报通过
                .withExternal()
                .source(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY)
                .target(TenderMainStatus.FINALLY_INSPECTED_PASSED)
                .event(TenderStateChangeEvent.FINALLY_INSPECTED_PASSED)
                .and()
                // 终验申报拒绝
                .withExternal()
                .source(TenderMainStatus.ON_FINALLY_INSPECTED_APPLY)
                .target(TenderMainStatus.FINALLY_INSPECTED_FAILED)
                .event(TenderStateChangeEvent.FINALLY_INSPECTED_FAILED)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "tenderStatePersister")
    public StateMachinePersister<ITenderStatus, TenderStateChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<ITenderStatus, TenderStateChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<ITenderStatus, TenderStateChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{} ==> {}", contextObj.getProjectId(), contextObj.getId());
            }

            @Override
            public StateMachineContext<ITenderStatus, TenderStateChangeEvent> read(Purchase contextObj) {
                return new DefaultStateMachineContext<>(TenderMainStatus.getNoNull(contextObj.getStatus()),
                        null, null, null, null, MACHINE_ID);
            }
        });
    }

}
