package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.TestValidStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderTestValidStatus;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * <p>
 * 标段测试验证状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@EnableStateMachine(name = TestValidStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class TestValidStateMachineBuilderImpl implements BaseStateMachineBuilder<Purchase, TestValidStateChangeEvent, TenderTestValidStatus> {

    public static final String MACHINE_ID = "testValidStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<TenderTestValidStatus, TestValidStateChangeEvent> build() throws StateMachineException {
        StateMachine<TenderTestValidStatus, TestValidStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：" + stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<TenderTestValidStatus, TestValidStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<TenderTestValidStatus, TestValidStateChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<TenderTestValidStatus, TestValidStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        builder.configureStates()
                .withStates()
                .initial(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO)
                .states(EnumSet.allOf(TenderTestValidStatus.class));

        builder.configureTransitions()
                .withExternal()
                .source(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO)
                .target(TenderTestValidStatus.TEST_VALID_INFO_AUDIT)
                .event(TestValidStateChangeEvent.SUBMIT_TEST_VALID_INFO)
                .and()
                .withExternal()
                .source(TenderTestValidStatus.TEST_VALID_INFO_AUDIT)
                .target(TenderTestValidStatus.TEST_VALID_INFO_PASSED)
                .event(TestValidStateChangeEvent.TEST_VALID_INFO_PASSED)
                .and()
                .withExternal()
                .source(TenderTestValidStatus.TEST_VALID_INFO_AUDIT)
                .target(TenderTestValidStatus.TEST_VALID_INFO_FAILED)
                .event(TestValidStateChangeEvent.TEST_VALID_INFO_FAILED)
                .and()
                .withExternal()
                .source(TenderTestValidStatus.TEST_VALID_INFO_FAILED)
                .target(TenderTestValidStatus.TEST_VALID_INFO_AUDIT)
                .event(TestValidStateChangeEvent.RESUBMIT_TEST_VALID_INFO)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "testValidStatePersister")
    public StateMachinePersister<TenderTestValidStatus, TestValidStateChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<TenderTestValidStatus, TestValidStateChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<TenderTestValidStatus, TestValidStateChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{} ==> {}", contextObj.getProjectId(), contextObj.getId());
            }

            @Override
            public StateMachineContext<TenderTestValidStatus, TestValidStateChangeEvent> read(Purchase contextObj) {
                TenderTestValidStatus testValidStatus = TenderTestValidStatus.getNoNull(contextObj.getTestValidStatus());
                return new DefaultStateMachineContext<>(testValidStatus, null, null, null, null, MACHINE_ID);
            }
        });
    }

}
