package com.hz.pm.api.common.statemachine.util;

import com.hz.pm.api.common.statemachine.builder.impl.SelfTestStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.SelfTestStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderSelfTestStatus;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.function.Function;

/**
 * <p>
 * 标段状态机工具类
 * </p>
 *
 * @author WendyYang
 * @since 17:06 2024/2/24
 */
@Slf4j
@Component
public class SelfTestStateMachineUtil implements AbstractStateMachineUtil<Purchase, SelfTestStateChangeEvent> {

    public static final String PURCHASE = "purchaseInfo";

    //==================================================================================================================

    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final StateMachinePersister<TenderSelfTestStatus, SelfTestStateChangeEvent, Purchase> stateMachinePersister;
    private final StateMachine<TenderSelfTestStatus, SelfTestStateChangeEvent> stateMachine;

    public SelfTestStateMachineUtil(SelfTestStateMachineBuilderImpl selfTestStateMachineBuilder, IPurchaseStatusChangeService purchaseStatusChangeService) {
        this.purchaseStatusChangeService = purchaseStatusChangeService;
        this.stateMachinePersister = selfTestStateMachineBuilder.stateMachinePersister();
        this.stateMachine = selfTestStateMachineBuilder.build();
    }

    @Override
    public Function<Purchase, Integer> statusFunction() {
        return Purchase::getSelfTestStatus;
    }

    @Override
    public void execute(Purchase purchase, SelfTestStateChangeEvent event) throws Exception {
        // 获取TO状态机
        PurchaseStatusChange change = new PurchaseStatusChange();
        change.setBeforeStatus(purchase.getSelfTestStatus());
        change.setEvent(event.name());
        change.setBidId(purchase.getId());
        change.setProjectId(purchase.getProjectId());
        change.setProjectCode(ProjectIdCodeCacheUtil.get(purchase.getProjectId()));
        change.setCreateOn(LocalDateTime.now());
        Message<SelfTestStateChangeEvent> message = MessageBuilder.withPayload(event)
                .setHeader(PURCHASE, purchase)
                .build();
        //初始化状态机
        stateMachinePersister.restore(stateMachine, purchase);
        stateMachine.sendEvent(message);
        change.setAfterStatus(purchase.getSelfTestStatus());
        purchaseStatusChangeService.save(change);
        log.info("调用状态机后的标段状态为：{} ==> {}", change.getBeforeStatus(), change.getAfterStatus());
    }

}
