package com.hz.pm.api.common.util;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.NumberUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 * BizUtils
 * </p>
 *
 * @author WendyYang
 * @since 17:32 2023/1/29
 */
public class BizUtils {

    private BizUtils() {

    }

    public static <T extends Collection<?>> void notEmpty(T objs, Consumer<T> consumer) {
        if (objs != null && !objs.isEmpty()) {
            consumer.accept(objs);
        }
    }

    public static <T extends Number> List<T> splitToNum(String str, Class<T> aClass) {
        if (StrUtil.isEmpty(str)) {
            return Collections.emptyList();
        }
        return Arrays.stream(str.split(StrPool.COMMA))
                .map(w -> NumberUtils.parseNumber(w, aClass))
                .collect(Collectors.toList());
    }

    public static List<Long> splitToLong(String str) {
        return splitToNum(str, Long.class);
    }


    public static <T> void notNull(T obj, Consumer<T> consumer) {
        if (obj != null) {
            consumer.accept(obj);
        }
    }

    public static void notBlank(String str, Consumer<String> consumer) {
        if (StrUtil.isNotBlank(str)) {
            consumer.accept(str);
        }
    }

    public static String uuid32() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 对象分组取第一条
     *
     * @param objs       对象集合
     * @param group      分组函数
     * @param comparator 比较器
     * @return java.util.Collection<T>
     * @author WendyYang
     **/
    public static <T> Collection<T> groupFirst(Collection<T> objs, Function<T, Object> group, Comparator<T> comparator) {
        return groupFirstMap(objs, group, comparator).values();
    }

    /**
     * 对象分组取第一条
     *
     * @param objs       对象集合
     * @param group      分组函数
     * @param comparator 比较器
     * @return java.util.Collection<T>
     * @author WendyYang
     **/
    public static <V, K> Map<K, V> groupFirstMap(Collection<V> objs, Function<V, K> group, Comparator<V> comparator) {
        return objs.stream().collect(Collectors.groupingBy(group,
                Collectors.collectingAndThen(toList(), w -> {
                    w.sort(comparator);
                    return w.get(0);
                })));
    }

    public static String inSqlJoin(Collection<String> strings) {
        return strings.stream().map(w -> "'" + w + "'").collect(Collectors.joining(StrPool.COMMA, StrPool.LEFT_BRACKET, StrPool.RIGHT_BRACKET));
    }

    public static String getProjectTypeName(String projectType) {
        if (Objects.nonNull(projectType)) {
            return ProjectTypeNewEnum.getDesc(projectType);
        }
        return StringUtils.EMPTY;
    }

    public static String computeRise(Integer numa, Integer numb) {
        if ((Objects.isNull(numb) || numb.equals(0)) &&
                (Objects.isNull(numa) || numa.equals(0))) {
            return "0";
        }
        if (Objects.isNull(numa) || numa.equals(0)) {
            return "-100";
        }
        if (Objects.isNull(numb) || numb.equals(0)) {
            return "100";
        }
        return BigDecimal.valueOf((numa - numb) * 100.0).divide(BigDecimal.valueOf(numb), 2, RoundingMode.CEILING)
                .stripTrailingZeros().toPlainString();
    }

    public static String buildFileJson(FileResultVO resultVo) {
        JSONArray jsonArray = new JSONArray();
        jsonArray.add(resultVo);
        return JSON.toJSONString(jsonArray);
    }

}
