package com.hz.pm.api.external;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.resource.ClassPathResource;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.common.util.EnvironmentUtil;
import com.hz.pm.api.external.model.dto.*;
import com.hz.pm.api.external.model.enumeration.MhDictType;
import com.hz.pm.api.meeting.entity.req.MeetingIdReq;
import com.hz.pm.api.open.model.vo.MeetingExpertToMhDTO;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoDTO;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

import static cn.hutool.core.date.DatePattern.NORM_DATETIME_FORMATTER;

/**
 * <p>
 * MhApiClient
 * </p>
 *
 * @author WendyYang
 * @since 20:41 2023/12/20
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class MhApiClient {

    private final EnvironmentUtil environmentUtil;

    @Value("${mh.api-host:}")
    private String mhApiHost;

    @Value("${mh.zwdd-work-notice.client-id:}")
    private String zwddWorkNoticeClientId;

    @Value("${mh.zwdd-work-notice.client-secret:}")
    private String zwddWorkNoticeClientSecret;

    private static final String UNIT_URL = "/sync/unit";
    private static final String USER_URL = "/sync/user";
    private static final String EXPERT_URL = "/sync/expert";
    private static final String COMPANY_URL = "/sync/company";
    private static final String PROJECT_URL = "/project_info/getProjectInfoList";

    private static final String ZWDD_MSG_URL = "/message/send";

    private static final String DICT_URL = "/dict_data/listByType";

    private static final String PURCHASE_NOTICE = "/buy_notice/saveBuyNoticeList";
    private static final String PURCHASE_NOTICE_GET = "/buy_notice/getBuyIdeaList";
    private static final String PUSH_MEETING_EXPERTS_DATA = "/meeting/saveData";
    private static final String CANCEL_MEETING_DATA = "/meeting/cancel";

    private static final String SYSTEM_REPLACE_INFO = "/deep_design/dzSystem";

    /**
     * 信创符合性测评报告相关接口
     */
    private static final String XCFHX_REPORT_LIST_UTL = "/access/getAccessList";

    private static final String XCFHX_REPORT_DETAIL_UTL = "/access/getInfoById";


    public MhRetDTO<List<MhUnitDTO>> queryUnits() {
        String requestUrl = mhApiHost + UNIT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUnitDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhCompanyDTO>> queryCompany() {
        String requestUrl = mhApiHost + COMPANY_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhCompanyDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhUserDTO>> queryUsers(LocalDateTime syncDateTime) {
        String requestUrl = mhApiHost + USER_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl += "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUserDTO>>>() {
        }, false);
    }

    public MhRetDTO<MhExpertDTO> queryExperts(LocalDateTime syncDateTime) {
        String requestUrl = mhApiHost + EXPERT_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl += "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<MhExpertDTO>>() {
        }, false);
    }

    public MhRetDTO<List<MhProjectDTO>> listProjects() {
        String requestUrl = mhApiHost + PROJECT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhProjectDTO>>>() {
        }, false);
    }

    public void publishPurchaseNotice(List<MhPurchaseNoticeDTO> notices) {
        if (environmentUtil.isDevEnv()) {
            return;
        }
        String requestUrl = mhApiHost + PURCHASE_NOTICE;
        String retBody = HttpUtil.post(requestUrl, JSONUtil.toJsonStr(notices));
        MhRetDTO<Object> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<Object>>() {
                }, false);
        if (!retObj.isOk()) {
            log.error("采购公告发布失败：{}", retBody);
            throw BizException.wrap("采购公告发布失败");
        }
    }

    public boolean sendZwddWorkNotice(MhZwddWorkNoticeDTO notice) {
        notice.setClientId(zwddWorkNoticeClientId);
        notice.setClientSecret(zwddWorkNoticeClientSecret);
        String retBody = HttpUtil.post(mhApiHost + ZWDD_MSG_URL, JSONUtil.toJsonStr(notice));
        JSONObject retObj = JSONUtil.parseObj(retBody);
        if (retObj.containsKey("code") && retObj.getInt("code") == 200) {
            return Boolean.TRUE;
        }
        log.warn("发送浙政钉工作通知失败：{}", retObj);
        return Boolean.FALSE;
    }

    public List<MhDictDTO> dictListByType(MhDictType dictType) {
        String retBody;
        if (!environmentUtil.isDevEnv()) {
            String url = mhApiHost + DICT_URL;
            retBody = HttpUtil.get(url + "?dictType=" + dictType);
        } else {
            retBody = new ClassPathResource("/response/ret-purchase-method.json").readUtf8Str();
        }
        MhRetDTO<List<MhDictDTO>> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<List<MhDictDTO>>>() {
                }, false);
        if (retObj.isOk()) {
            return retObj.getData();
        }
        log.error("获取字典信息失败：{} {}", dictType, retBody);
        throw BizException.wrap("获取字典【%s】失败", dictType.getVal());
    }

    public List<MhPurchaseIntentionDTO> listPurchaseNotice(Long unitId) {
        String retBody;
        if (!environmentUtil.isDevEnv()) {
            String url = mhApiHost + PURCHASE_NOTICE_GET;
            retBody = HttpUtil.get(url + "?unitId=" + unitId);
        } else {
            retBody = new ClassPathResource("/response/ret-purchase-intention.json").readUtf8Str();
        }
        MhRetDTO<List<MhPurchaseIntentionDTO>> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<List<MhPurchaseIntentionDTO>>>() {
                }, false);
        if (retObj.isOk()) {
            return retObj.getData();
        }
        log.error("获取采购意向失败：{} {}", unitId, retBody);
        throw BizException.wrap("获取采购意向失败");
    }

    public List<MhXcfhxReportListDTO> pageXcfhxTestReport() {
        String retBody;
        if (!environmentUtil.isDevEnv()) {
            String url = mhApiHost + XCFHX_REPORT_LIST_UTL;
            retBody = HttpUtil.get(url);
        } else {
            retBody = new ClassPathResource("/response/ret-xcfhx-report.json").readUtf8Str();
        }
        MhRetDTO<List<MhXcfhxReportListDTO>> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<List<MhXcfhxReportListDTO>>>() {
                }, false);
        if (retObj.isOk()) {
            return retObj.getData();
        }
        log.error("获取测评报告失败：{}", retBody);
        throw BizException.wrap("获取测评报告失败");
    }

    public String pushMeetingAndExpertsDataToMh(MeetingExpertToMhDTO data) {
        if (environmentUtil.isDevEnv()) {
            return StrUtil.EMPTY;
        }
        String requestUrl = mhApiHost + PUSH_MEETING_EXPERTS_DATA;
        log.info("推送会议专家信息到MH：{}", data);
        String retBody = HttpUtil.post(requestUrl, JSONUtil.toJsonStr(data));
        MhRetDTO<JSONObject> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<JSONObject>>() {
                }, false);
        if (!retObj.isOk()) {
            log.error("会议专家信息推送失败：{}", retBody);
            return StrUtil.EMPTY;
        }
        // 返回信产会议签到页面地址
        return retObj.getData().getStr("url");
    }

    public void cancelMeetingToMh(Long meetingId) {
        if (environmentUtil.isDevEnv()) {
            return;
        }
        String requestUrl = mhApiHost + CANCEL_MEETING_DATA;
        log.info("取消会议推送到MH：{}", meetingId);
        MeetingIdReq meetingIdReq = new MeetingIdReq();
        meetingIdReq.setMeetingId(meetingId);
        String retBody = HttpUtil.post(requestUrl, JSONUtil.toJsonStr(meetingIdReq));
        MhRetDTO<Object> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<Object>>() {
                }, false);
        if (!retObj.isOk()) {
            log.error("取消会议信息推送失败：{}", retBody);
        }
    }

    public List<MhSystemReplaceInfoDTO> querySystemReplaceInfos() {
        String retBody;
        if (!environmentUtil.isDevEnv()) {
            String url = mhApiHost + SYSTEM_REPLACE_INFO;
            retBody = HttpUtil.get(url);
        } else {
            retBody = new ClassPathResource("/response/ret-system-replace-info.json").readUtf8Str();
        }
        MhRetDTO<List<MhSystemReplaceInfoDTO>> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<List<MhSystemReplaceInfoDTO>>>() {
                }, false);
        if (retObj.isOk()) {
            return retObj.getData();
        }
        log.error("系统改造计划获取失败：{}", retBody);
        throw BizException.wrap("系统改造计划获取失败");
    }

}
