package com.hz.pm.api.external.sms;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hz.pm.api.external.sms.dto.SmsDTO;
import com.hz.pm.api.external.sms.vo.SmsReceipt;
import com.hz.pm.api.external.sms.vo.SmsReplyDTO;
import com.hz.pm.api.external.sms.vo.SmsSendDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * MhSmsClient
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/4/25
 */
@Slf4j
public class MhSmsClient {

    private static volatile MhSmsClient client;

    private static volatile boolean smsSendOpen = Boolean.TRUE;

    private static final String SMS_HOST_KEY = "sms-client.host";
    private static final String SMS_OPEN_KEY = "sms-client.open";

    private static final Object LOCK = new Object();

    public static MhSmsClient getClient() {
        if (smsSendOpen && client == null) {
            synchronized (LOCK) {
                if (client == null) {
                    Environment environment = SpringUtil.getBean(Environment.class);
                    smsSendOpen = environment.getProperty(SMS_OPEN_KEY, Boolean.class, Boolean.TRUE);
                    if (smsSendOpen) {
                        String smsHost = environment.getProperty(SMS_HOST_KEY);
                        Assert.notBlank(smsHost, "短信服务地址不能为空");
                        client = new MhSmsClient(smsHost);
                    }
                }
            }
        }
        Assert.isTrue(smsSendOpen, "短信服务未开启");
        return client;
    }

    //==================================================================================================================

    private MhSmsClient(String smsUrl) {
        this.smsUrl = smsUrl;
    }

    /**
     * 短信服务地址前缀
     */
    private final String smsUrl;
    /**
     * 获取短信回复结果
     */
    private static final String SMS_REPLY = "/sms/reply";
    /**
     * 短信发送
     */
    private static final String SMS_SEND = "/sms/send";
    /**
     * 短信状态
     */
    private static final String SMS_RECEIPT = "/sms/status";

    /**
     * 短信回复
     *
     * @param result \
     * @return \
     */
    public SmsReplyDTO smsReply(String result) {
        String refreshUrl = smsUrl + SMS_REPLY;
        Map<String, Object> map = new HashMap<>();
        map.put("result", result);
        String responseResult = HttpUtil.post(refreshUrl, map);
        JSONObject responseJson = JSON.parseObject(responseResult, JSONObject.class);
        String fileData = responseJson.getString("data");
        return JSONUtil.toBean(fileData, new TypeReference<SmsReplyDTO>() {
        }, false);
    }

    /**
     * 短信发送
     *
     * @return \
     */
    public SmsDTO<SmsSendDTO> smsSend(String content, Collection<String> phones) {
        String phonesSplit = String.join(",", phones);
        String refreshUrl = smsUrl + SMS_SEND;
        Map<String, Object> map = new HashMap<>();
        map.put("content", content);
        map.put("phones", phonesSplit);
        String responseResult = HttpUtil.post(refreshUrl, JSON.toJSONString(map));
        return JSONUtil.toBean(responseResult, new TypeReference<SmsDTO<SmsSendDTO>>() {
        }, false);
    }

    /**
     * 短信发送
     *
     * @return \
     */
    public SmsDTO<SmsSendDTO> smsSend(String content, String phone) {
        return smsSend(content, Collections.singletonList(phone));

    }

    /**
     * 短信回执
     *
     * @param result \
     * @return \
     */
    public SmsReceipt smsReceipt(String result) {
        String refreshUrl = smsUrl + SMS_RECEIPT;
        HashMap<String, Object> map = new HashMap<>();
        map.put("result", result);
        String responseResult = HttpUtil.post(refreshUrl, map);
        JSONObject responseJson = JSON.parseObject(responseResult, JSONObject.class);
        String fileData = responseJson.getString("data");
        JSONObject jsonObject = JSON.parseObject(fileData);
        Map<String, Object> dataMap = jsonObject.getInnerMap();
        if (dataMap.get("result") instanceof Boolean) {
            return SmsReceipt.builder().error(dataMap.get("error").toString()).build();
        }
        return JSONObject.parseObject(fileData, SmsReceipt.class);
    }

}
