package com.hz.pm.api.external.todo;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.http.Method;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.util.EnvironmentUtil;
import com.hz.pm.api.external.todo.dto.MhTodoRequestDTO;
import com.hz.pm.api.external.todo.dto.MhTodoRetDTO;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.sys.model.entity.MhTodoRecord;
import com.hz.pm.api.sys.service.IMhTodoRecordService;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * @author 王仁康
 * @date 2024-01-02 16:55:51
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class MHTodoClient {

    /**
     * 发送待办接口地址
     */
    private static final String TODO_ADD = "/single_wk_handle/addWkhandle";


    /**
     * 完成待办接口地址
     */
    private static final String TODO_COMPLETE = "/single_wk_handle/completeHandle";

    private static final String WK_HANDLE_ID = "wkhandleId";

    private static final TypeReference<MhTodoRetDTO> TYPE_REF = new TypeReference<MhTodoRetDTO>() {
    };

    //==================================================================================================================

    private final IMhTodoRecordService todoRecordService;
    private final EnvironmentUtil environmentUtil;


    /**
     * 外部系统域名
     */
    @Value("${mh.api-host:}")
    private String mhApiHost;

    @Transactional(rollbackFor = Exception.class)
    public void addTodo(Object paramObj, UserFullInfoDTO receiver, MHTodoTypeEnum type, String bizId, String content) {
        if (environmentUtil.isDevEnv()) {
            return;
        }
        String url = mhApiHost + TODO_ADD;
        String paramObjs = JSON.toJSONString(paramObj);
        MhTodoRequestDTO todoReq = MhTodoRequestDTO.builder()
                .wkContent(content)
                .wkType(type.name())
                .params(paramObjs)
                .wkUser(receiver.getMhUserId())
                .build();
        log.info("添加新创待办：{}", todoReq);
        String responseResult = HttpUtil.post(url, JSON.toJSONString(todoReq));
        MhTodoRetDTO retData = JSON.parseObject(responseResult, TYPE_REF);
        if (retData.getCode() != 200 || StrUtil.isBlank(retData.getData())) {
            log.warn("发送待办失败：{} {}", todoReq, retData);
            throw BizException.wrap("发送待办失败");
        }
        MhTodoRecord todo = new MhTodoRecord();
        todo.setMhTodoId(retData.getData());
        todo.setContent(content);
        todo.setParamObj(paramObjs);
        todo.setBizId(bizId);
        todo.setType(type.name());
        todo.setUserId(receiver.getUserId());
        todo.setToMhUserId(receiver.getMhUserId());
        todo.setStatus(MhTodoRecord.TodoStatusEnum.PENDING.getCode());
        todoRecordService.save(todo);
    }

    /**
     * 完成待办
     *
     * @param wkhandleId 待办ID
     */
    private void complete(String wkhandleId) {
        if (environmentUtil.isDevEnv()) {
            return;
        }
        String url = mhApiHost + TODO_COMPLETE;
        // 发送PUT请求
        HttpRequest request = HttpUtil.createRequest(Method.PUT, url)
                .form(WK_HANDLE_ID, wkhandleId);
        try (HttpResponse response = request.execute()) {
            MhTodoRetDTO retData = JSON.parseObject(response.body(), TYPE_REF);
            boolean state = Boolean.parseBoolean(retData.getData());
            if (!state) {
                log.error("待办完成失败：{} {}", wkhandleId, retData);
            }
        }

    }


    public void completeTodo(MHTodoTypeEnum todoType, Collection<String> bizIds) {
        if (environmentUtil.isDevEnv()) {
            return;
        }
        LambdaQueryWrapper<MhTodoRecord> query = Wrappers.lambdaQuery(MhTodoRecord.class)
                .eq(MhTodoRecord::getType, todoType.name())
                .in(MhTodoRecord::getBizId, bizIds)
                .eq(MhTodoRecord::getStatus, MhTodoRecord.TodoStatusEnum.PENDING.getCode());
        List<MhTodoRecord> todos = todoRecordService.list(query);
        if (todos.isEmpty()) {
            return;
        }
        for (MhTodoRecord todo : todos) {
            complete(todo.getMhTodoId());
        }
        LambdaUpdateWrapper<MhTodoRecord> update = Wrappers.lambdaUpdate(MhTodoRecord.class)
                .in(MhTodoRecord::getId, CollUtils.fieldList(todos, MhTodoRecord::getId))
                .set(MhTodoRecord::getStatus, MhTodoRecord.TodoStatusEnum.HANDED.getCode());
        todoRecordService.update(update);
    }

    public void completeTodo(MHTodoTypeEnum todoType, String bizId) {
        completeTodo(todoType, Collections.singletonList(bizId));
    }

}