package com.hz.pm.api.meeting.manage;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.expert.entity.ExpertAvoidCompany;
import com.hz.pm.api.expert.entity.ExpertGovBusinessStrip;
import com.hz.pm.api.expert.entity.ExpertUserFullInfo;
import com.hz.pm.api.expert.service.IExpertAvoidCompanyService;
import com.hz.pm.api.expert.service.IExpertGovBusinessStripService;
import com.hz.pm.api.expert.service.IExpertIntentionWorkRegionService;
import com.hz.pm.api.expert.service.IExpertUserFullInfoService;
import com.hz.pm.api.meeting.builder.ExpertInviteBuilder;
import com.hz.pm.api.meeting.entity.domain.ExpertInviteRule;
import com.hz.pm.api.meeting.entity.domain.Meeting;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.entity.dto.*;
import com.hz.pm.api.meeting.entity.enumeration.AvoidTypeEnum;
import com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.hz.pm.api.meeting.entity.enumeration.MeetingStatusEnum;
import com.hz.pm.api.meeting.helper.ExpertInviteHelper;
import com.hz.pm.api.meeting.helper.MeetingNotifyHelper;
import com.hz.pm.api.meeting.service.IExpertInviteRuleService;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.hz.pm.api.meeting.service.IMeetingService;
import com.hz.pm.api.meta.helper.TagCache;
import com.hz.pm.api.meta.model.entity.ExpertDictionary;
import com.hz.pm.api.meta.model.entity.ExpertTag;
import com.hz.pm.api.meta.service.IExpertDictionaryService;
import com.hz.pm.api.meta.service.IExpertTagService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.Predicate;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum.*;
import static com.hz.pm.api.meeting.helper.ExpertInviteHelper.getExpertInviteRule;
import static com.hz.pm.api.meeting.helper.ExpertRandomInviteAlgorithm.*;

/**
 * <p>
 * 专家邀请管理
 * </p>
 *
 * @author WendyYang
 * @since 18:05 2022/8/8
 */
@Component
@RequiredArgsConstructor
public class ExpertInviteManage {

    private final IExpertDictionaryService expertDictionaryService;
    private final TagCache tagCache;
    private final IExpertTagService expertTagService;
    private final IExpertIntentionWorkRegionService workRegionService;
    private final IExpertInviteRuleService inviteRuleService;
    private final IMeetingExpertService meetingExpertService;
    private final ExpertInviteHelper expertInviteHelper;
    private final IExpertUserFullInfoService expertUserFullInfoService;
    private final IMeetingService meetingService;
    private final IExpertAvoidCompanyService expertAvoidCompanyService;
    private final MeetingNotifyHelper meetingNotifyHelper;
    private final IExpertGovBusinessStripService expertGovBusinessStripService;

    @Value("#{randomInviteProperties.recentMeetingCount}")
    private Integer recentMeetingCount;

    @Value("#{randomInviteProperties.recentDays}")
    private Integer recentDays;

    private static final Predicate<Collection<?>> COLL_EMPTY = (coll) -> coll != null && coll.isEmpty();

    private static LambdaQueryWrapper<ExpertUserFullInfo> buildBaseExpertQuery() {
        return Wrappers.lambdaQuery(ExpertUserFullInfo.class)
                .select(ExpertUserFullInfo::getUserId,
                        ExpertUserFullInfo::getId,
                        ExpertUserFullInfo::getCompany,
                        ExpertUserFullInfo::getExpertName,
                        ExpertUserFullInfo::getCompanyUniqCode,
                        ExpertUserFullInfo::getPhoneNo);
    }

    private static void buildAvoidCompanyAndBusinessStrip(LambdaQueryWrapper<ExpertUserFullInfo> query, List<String> units, List<String> strips) {
        if (CollUtil.isNotEmpty(units)) {
            String unitStr = BizUtils.inSqlJoin(units);
            query.notExists("select 1 from expert_avoid_company eac where eac.user_id = nd_expert_user_full_info.user_id" +
                    " and company_uniq_code in " + unitStr);
        }
        if (CollUtil.isNotEmpty(strips)) {
            String orgStr = BizUtils.inSqlJoin(strips);
            query.notExists("select 1 from expert_gov_business_strip egbs where egbs.expert_user_id = nd_expert_user_full_info.user_id" +
                    " and business_strip_code in " + orgStr);
        }
    }

    /**
     * 获取一周内被抽中并同意参会的专家ID
     *
     * @param agreeCnt 一周内被抽中并同意参会次数
     * @param sTime    会议开始时间
     * @param days     天数
     * @return java.util.List<java.lang.Long>
     * @author WendyYang
     **/
    private List<Long> listAgreedUserIdByRecentMeetings(int agreeCnt, int days, LocalDateTime sTime) {
        if (agreeCnt == 0 || days == 0) {
            return Collections.emptyList();
        }
        LocalDateTime beginLimit = sTime.minusDays(days);
        return meetingExpertService.listAgreeExpertIdByRecentDaysAndAgreeCount(agreeCnt, beginLimit, sTime);
    }

    /**
     * 增加专家层级限制
     *
     * @param query 查询
     * @param rule  随机邀请规则
     */
    private static void addRegionLimit(LambdaQueryWrapper<ExpertUserFullInfo> query, RandomInviteRuleDTO rule) {
        if (CollUtil.isNotEmpty(rule.getExpertRegionList())) {
            String inSqlParam = rule.getExpertRegionList().stream()
                    .map(w -> "('" + w.getRegionCode() + "'," + w.getRegionLevel() + ")")
                    .collect(Collectors.joining(",", "(", ")"));
            query.apply("(region_code, region_level) in " + inSqlParam);
        }
    }

    /**
     * 获取满足履职意向地的专家ID
     *
     * @param rule 抽取规则
     * @return 专家ID
     * @author WendyYang
     **/
    private List<Long> expertIdsByRegion(RandomInviteRuleDTO rule) {
        String regionCode;
        Integer regionLevel;
        if ((regionCode = rule.getIntentionRegionCode()) == null
                || (regionLevel = rule.getIntentionRegionLevel()) == null) {
            return null;
        }
        return workRegionService.userIdsMatchIntentionRegion(regionCode, regionLevel);
    }


    /**
     * 如果抽取回避单位和专家回避单位一致，同样需要回避该专家
     *
     * @param unitCodeList /
     * @return /
     */
    private List<Long> avoidCompanyExpertIds(List<String> unitCodeList) {
        if (CollUtil.isEmpty(unitCodeList)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<ExpertAvoidCompany> query = Wrappers.lambdaQuery(ExpertAvoidCompany.class)
                .select(ExpertAvoidCompany::getUserId)
                .in(ExpertAvoidCompany::getCompanyUniqCode, unitCodeList);
        List<ExpertAvoidCompany> expertAvoidCompanyList = expertAvoidCompanyService.list(query);
        return CollUtils.fieldList(expertAvoidCompanyList, ExpertAvoidCompany::getUserId);
    }

    private MergeExpertIdDTO mergeExpertIdsByCondition(RandomInviteRuleDTO rule, AvoidRuleDTO avoidInfo) {
        // 处理履职意向地
        List<Long> expertIdsByIntentionRegion = expertIdsByRegion(rule);
        if (COLL_EMPTY.evaluate(expertIdsByIntentionRegion)) {
            return MergeExpertIdDTO.noExpert();
        }
        // 处理专家标签
        List<Long> expertIdsByTag = expertIdsByTag(rule);
        if (COLL_EMPTY.evaluate(expertIdsByTag)) {
            return MergeExpertIdDTO.noExpert();
        }
        // 处理专家字典
        List<Long> expertIdsByDict = expertIdsByDict(rule);
        if (COLL_EMPTY.evaluate(expertIdsByDict)) {
            return MergeExpertIdDTO.noExpert();
        }
        // 处理专家回避单位 与 抽取回避单位是否相同
        List<Long> avoidExpertIdsByUnit = avoidCompanyExpertIds(avoidInfo.getAvoidUnitIdList());

        MergeExpertIdDTO merge = new MergeExpertIdDTO();
        // 聚合用户ID
        List<Long> expertIdsIn = new ArrayList<>();
        if (ObjectUtils.anyNotNull(expertIdsByDict, expertIdsByTag, expertIdsByIntentionRegion)) {
            Optional.ofNullable(expertIdsByDict).ifPresent(expertIdsIn::addAll);
            Optional.ofNullable(expertIdsByTag).ifPresent(item -> {
                if (expertIdsIn.isEmpty()) {
                    expertIdsIn.addAll(expertIdsByTag);
                } else {
                    expertIdsIn.removeIf(w -> !expertIdsByTag.contains(w));
                }
            });
            Optional.ofNullable(expertIdsByIntentionRegion).ifPresent(item -> {
                if (expertIdsIn.isEmpty()) {
                    expertIdsIn.addAll(expertIdsByIntentionRegion);
                } else {
                    expertIdsIn.removeIf(w -> !expertIdsByIntentionRegion.contains(w));
                }
            });
            if (expertIdsIn.isEmpty()) {
                return MergeExpertIdDTO.noExpert();
            }
            merge.setExpertIdsIn(expertIdsIn);
        }
        if (CollUtil.isNotEmpty(avoidExpertIdsByUnit)) {
            if (expertIdsIn.isEmpty()) {
                merge.setExpertIdsNotIn(avoidExpertIdsByUnit);
            } else {
                expertIdsIn.removeIf(avoidExpertIdsByUnit::contains);
                if (expertIdsIn.isEmpty()) {
                    return MergeExpertIdDTO.noExpert();
                }
            }
        }
        merge.setSkip(Boolean.FALSE);
        return merge;
    }

    /**
     * 根据专家标签获取满足的专家ID
     *
     * @param rule 抽取规则
     * @return {@link ExpertIdFilterDTO}
     * @author WendyYang
     **/
    private List<Long> expertIdsByTag(RandomInviteRuleDTO rule) {
        if (CollectionUtils.isEmpty(rule.getExpertTags())) {
            return null;
        }
        LambdaQueryWrapper<ExpertTag> query = Wrappers.lambdaQuery(ExpertTag.class)
                .select(ExpertTag::getUserId, ExpertTag::getExpertInfoField);
        rule.getExpertTags().forEach(tag -> {
            List<String> codes = tag.getTagCodes().stream()
                    .map(tagCache::listChildrenCodes)
                    .flatMap(Collection::stream)
                    .collect(Collectors.toList());
            query.or(wrapper -> wrapper.eq(ExpertTag::getExpertInfoField, tag.getExpertTag())
                    .in(ExpertTag::getTagCode, codes));
        });
        List<ExpertTag> expertTagList = expertTagService.list(query);
        return expertTagList.stream()
                .collect(Collectors.groupingBy(ExpertTag::getUserId, Collectors.collectingAndThen(Collectors.toList(),
                        tags -> CollUtils.fieldList(tags, ExpertTag::getExpertInfoField).size())))
                .entrySet().stream().filter(w -> w.getValue() == rule.getExpertTags().size())
                .map(Map.Entry::getKey).collect(Collectors.toList());
    }

    /**
     * 根据专家字典获取满足的专家ID
     *
     * @param rule 抽取规则
     * @return 专家ID
     * @author WendyYang
     **/
    private List<Long> expertIdsByDict(RandomInviteRuleDTO rule) {
        if (CollUtil.isEmpty(rule.getExpertDicts())) {
            return null;
        }
        LambdaQueryWrapper<ExpertDictionary> query = Wrappers.lambdaQuery(ExpertDictionary.class)
                .select(ExpertDictionary::getUserId, ExpertDictionary::getExpertInfoField);
        for (ExpertDictChooseDTO dict : rule.getExpertDicts()) {
            query.or(wrapper -> wrapper.eq(ExpertDictionary::getExpertInfoField, dict.getExpertDict())
                    .in(ExpertDictionary::getDictionaryCode, dict.getDictCodes()));
        }
        List<ExpertDictionary> expertDictionaryList = expertDictionaryService.list(query);
        return expertDictionaryList.stream()
                .collect(Collectors.groupingBy(ExpertDictionary::getUserId,
                        Collectors.collectingAndThen(Collectors.toList(),
                                dicts -> CollUtils.fieldList(dicts, ExpertDictionary::getExpertInfoField).size())))
                .entrySet().stream().filter(w -> w.getValue() == rule.getExpertDicts().size())
                .map(Map.Entry::getKey).collect(Collectors.toList());
    }


    /**
     * 专家抽取（随机）
     *
     * @param avoidRule        回避信息
     * @param randomRule       抽取规则
     * @param invitedExpertIds 指定抽取专家ID
     * @param meetingCreateOn  会议创建时间
     * @return 满足抽取条件的专家
     * @author WendyYang
     **/
    public ExpertChoseDTO expertInviteByRandomRule(AvoidRuleDTO avoidRule,
                                                   RandomInviteRuleDTO randomRule,
                                                   List<Long> invitedExpertIds,
                                                   LocalDateTime sTime,
                                                   LocalDateTime eTime,
                                                   LocalDateTime meetingCreateOn) {
        ExpertChoseDTO result = new ExpertChoseDTO(new ArrayList<>(), 0);
        MergeExpertIdDTO merge = mergeExpertIdsByCondition(randomRule, avoidRule);
        if (merge.isSkip()) {
            return result;
        }
        Set<String> avoidUnitCodes = new HashSet<>();
        BizUtils.notEmpty(avoidRule.getAvoidUnitIdList(), avoidUnitCodes::addAll);

        LambdaQueryWrapper<ExpertUserFullInfo> query = buildBaseExpertQuery();
        AvoidTypeEnum avoidType = avoidRule.getAvoidType() == null ? AvoidTypeEnum.NONE : AvoidTypeEnum.getByCode(avoidRule.getAvoidType());
        if (CollUtil.isNotEmpty(invitedExpertIds)) {
            buildAvoidUnitStripForQuery(invitedExpertIds, avoidType, avoidUnitCodes, query);
        }
        // 处理专家层级
        addRegionLimit(query, randomRule);
        // 回避信息
        query.notIn(!avoidUnitCodes.isEmpty(), ExpertUserFullInfo::getCompanyUniqCode, avoidUnitCodes);
        // 处理回避单位与回避条线
        buildAvoidCompanyAndBusinessStrip(query, avoidRule.getAvoidUnitIdList(), avoidRule.getAvoidOrgIdList());
        Set<Long> expertIdsIn = new HashSet<>();
        Set<Long> expertIdsNotIn = new HashSet<>();
        if (CollUtil.isNotEmpty(merge.getExpertIdsIn())) {
            expertIdsIn.addAll(merge.getExpertIdsIn());
        } else if (CollUtil.isNotEmpty(merge.getExpertIdsNotIn())) {
            expertIdsNotIn.addAll(merge.getExpertIdsNotIn());
        }
        // 处理回避专家次数
        BizUtils.notNull(avoidRule.getWeekInviteCount(), weekInviteCount -> {
            List<Long> tmpExpertIdsNotIn = listAgreedUserIdByRecentMeetings(weekInviteCount, recentDays, meetingCreateOn);
            expertIdsNotIn.addAll(tmpExpertIdsNotIn);
        });

        if (!expertIdsIn.isEmpty()) {
            BizUtils.notEmpty(avoidRule.getExpertIds(), w -> expertIdsIn.removeIf(w::contains));
            BizUtils.notEmpty(invitedExpertIds, w -> expertIdsIn.removeIf(w::contains));
            if (expertIdsIn.isEmpty()) {
                return result;
            }
            // 过滤掉已参加会议的专家
            List<Long> lockedExpertIds = expertInviteHelper.listInvitedExpertIds(sTime, eTime);
            expertIdsIn.removeIf(lockedExpertIds::contains);
            if (expertIdsIn.isEmpty()) {
                return result;
            }
        } else if (CollUtil.isNotEmpty(avoidRule.getExpertIds()) || CollUtil.isNotEmpty(invitedExpertIds)) {
            Set<Long> tmpExpert = expertInviteHelper.getAvoidExpert(invitedExpertIds, avoidRule, sTime, eTime);
            expertIdsNotIn.addAll(tmpExpert);
        } else {
            Set<Long> tmpNotInUserIds = expertInviteHelper.listExpertLeaveOrInvited(sTime, eTime);
            expertIdsNotIn.addAll(tmpNotInUserIds);
        }
        if (!expertIdsIn.isEmpty()) {
            query.in(ExpertUserFullInfo::getUserId, expertIdsIn);
        } else if (!expertIdsNotIn.isEmpty()) {
            query.notIn(ExpertUserFullInfo::getUserId, expertIdsNotIn);
        }
        // 开始进行专家抽取
        List<ExpertUserFullInfo> userInfoList = expertUserFullInfoService.list(query);
        if (userInfoList.isEmpty()) {
            return result;
        }
        switch (avoidType) {
            case NONE:
                result.setExperts(inviteRandom(userInfoList, expertsByRecentMeeting(), randomRule.getCount()));
                result.setTotal(userInfoList.size());
                break;
            case CURR_UNIT:
                Map<String, List<ExpertUserFullInfo>> unitGroup = CollUtils.group(userInfoList, ExpertUserFullInfo::getCompanyUniqCode);
                result.setExperts(inviteGroupByUnit(unitGroup, expertsByRecentMeeting(), randomRule.getCount()));
                result.setTotal(unitGroup.size());
                break;
            case CURR_STRIP:
                List<Long> userIds = CollUtils.fieldList(userInfoList, ExpertUserFullInfo::getUserId);
                List<ExpertGovBusinessStrip> expertStrips = expertGovBusinessStripService.listByUserIds(userIds);
                if (!expertStrips.isEmpty()) {
                    Map<String, List<ExpertGovBusinessStrip>> stripGroup = CollUtils.group(expertStrips, ExpertGovBusinessStrip::getBusinessStripCode);
                    result.setExperts(inviteGroupByStrip(stripGroup, userInfoList, expertsByRecentMeeting(), randomRule.getCount()));
                    result.setTotal(stripGroup.size());
                } else {
                    result.setTotal(0);
                }
                break;
        }
        return result;
    }

    private void buildAvoidUnitStripForQuery(List<Long> invitedExpertIds, AvoidTypeEnum avoidType, Set<String> avoidUnitCodes, LambdaQueryWrapper<ExpertUserFullInfo> query) {
        if (AvoidTypeEnum.CURR_UNIT.equals(avoidType)) {
            List<ExpertUserFullInfo> tmpInvitedExperts = expertUserFullInfoService.listByUserIds(invitedExpertIds);
            Set<String> tmpUnitCodes = CollUtils.fieldSet(tmpInvitedExperts, ExpertUserFullInfo::getCompanyUniqCode);
            avoidUnitCodes.addAll(tmpUnitCodes);
        } else if (AvoidTypeEnum.CURR_STRIP.equals(avoidType)) {
            List<ExpertGovBusinessStrip> expertStrips = expertGovBusinessStripService.listByUserIds(invitedExpertIds);
            if (CollUtil.isNotEmpty(expertStrips)) {
                Set<String> businessStripCodes = CollUtils.fieldSet(expertStrips, ExpertGovBusinessStrip::getBusinessStripCode);
                query.notExists("select 1 from expert_gov_business_strip egbs " +
                        "where egbs.expertUserId = nd_expert_user_full_info.user_id " +
                        "and egbs.business_strip_code in " + BizUtils.inSqlJoin(businessStripCodes));
            }
        }
    }

    /**
     * 专家替换、补充
     *
     * @param avoidRule      回避信息
     * @param randomRule     随机抽取
     * @param invitedExperts 已抽取人员
     * @param count          抽取数量
     * @param msTime         会议开始时间
     * @param meTime         会议结束时间
     * @param reInvite       邀请已拒绝
     * @return {@link ExpertChoseDTO}
     * @author WendyYang
     **/
    public ExpertChoseDTO expertReplaceByRandomRule(AvoidRuleDTO avoidRule,
                                                    RandomInviteRuleDTO randomRule,
                                                    Collection<MeetingExpert> invitedExperts,
                                                    Integer count,
                                                    LocalDateTime msTime,
                                                    LocalDateTime meTime,
                                                    LocalDateTime tsTime,
                                                    boolean reInvite) {
        ExpertChoseDTO result = new ExpertChoseDTO(new ArrayList<>(), 0);
        // 合并标签、字典
        MergeExpertIdDTO merge = mergeExpertIdsByCondition(randomRule, avoidRule);
        if (merge.isSkip()) {
            return result;
        }
        Set<Long> expertIdsIn = new HashSet<>();
        Set<Long> expertIdsNotIn = new HashSet<>();
        if (CollUtil.isNotEmpty(merge.getExpertIdsIn())) {
            expertIdsIn.addAll(merge.getExpertIdsIn());
        } else if (CollUtil.isNotEmpty(merge.getExpertIdsNotIn())) {
            expertIdsNotIn.addAll(merge.getExpertIdsNotIn());
        }
        LambdaQueryWrapper<ExpertUserFullInfo> query = buildBaseExpertQuery();
        // 设置回避单位
        Set<String> avoidUnitCodes = new HashSet<>(avoidRule.getAvoidUnitIdList());
        // 处理回避单位与回避条线
        buildAvoidCompanyAndBusinessStrip(query, avoidRule.getAvoidUnitIdList(), avoidRule.getAvoidOrgIdList());
        // 处理回避专家次数
        BizUtils.notNull(avoidRule.getWeekInviteCount(), weekInviteCount -> {
            List<Long> tmpExpertIdsNotIn = listAgreedUserIdByRecentMeetings(weekInviteCount, recentDays, msTime);
            expertIdsNotIn.addAll(tmpExpertIdsNotIn);
        });
        // 处理专家层级
        addRegionLimit(query, randomRule);

        if (!expertIdsIn.isEmpty()) {
            if (CollectionUtils.isNotEmpty(avoidRule.getExpertIds())) {
                expertIdsIn.removeIf(w -> avoidRule.getExpertIds().contains(w));
                if (expertIdsIn.isEmpty()) {
                    // 字典、标签、履职意向地筛选出的专家ID移除需要回避的专家ID、如果为空则说明没有符合条件的
                    return result;
                }
            }
            List<Long> lockedExpertIds = expertInviteHelper.listInvitedExpertIds(msTime, meTime);
            expertIdsIn.removeIf(lockedExpertIds::contains);
            if (expertIdsIn.isEmpty()) {
                return result;
            }
        } else if (CollUtil.isNotEmpty(avoidRule.getExpertIds())) {
            Set<Long> notInExpertIds = expertInviteHelper.getAvoidExpert(avoidRule.getExpertIds(), null, msTime, meTime);
            expertIdsNotIn.addAll(notInExpertIds);
        } else {
            Set<Long> notInUserIds = expertInviteHelper.listExpertLeaveOrInvited(msTime, meTime);
            expertIdsNotIn.addAll(notInUserIds);
        }
        Map<ExpertAttendStatusEnum, List<MeetingExpert>> expertGroupByStatus = invitedExperts.stream()
                .collect(Collectors.groupingBy(w -> ExpertAttendStatusEnum.getByCode(w.getStatus())));
        // 回避同单位其他专家
        AvoidTypeEnum avoidType = avoidRule.getAvoidType() == null ? AvoidTypeEnum.NONE : AvoidTypeEnum.getByCode(avoidRule.getAvoidType());
        List<MeetingExpert> agreeOrNoticing = new ArrayList<>();
        BizUtils.notEmpty(expertGroupByStatus.get(AGREED), agreeOrNoticing::addAll);
        BizUtils.notEmpty(expertGroupByStatus.get(NOTICING), agreeOrNoticing::addAll);
        List<Long> agreeOrNoticingExpertIds = CollUtils.fieldList(agreeOrNoticing, MeetingExpert::getExpertId);
        if (!agreeOrNoticingExpertIds.isEmpty()) {
            expertIdsNotIn.addAll(agreeOrNoticingExpertIds);
            buildAvoidUnitStripForQuery(agreeOrNoticingExpertIds, avoidType, avoidUnitCodes, query);
        }
        // 已请假的专家不再抽取
        List<MeetingExpert> expertsOnLeave = expertGroupByStatus.get(ON_LEAVE);
        if (CollUtil.isNotEmpty(expertsOnLeave)) {
            expertIdsNotIn.addAll(CollUtils.fieldList(expertsOnLeave, MeetingExpert::getExpertId));
        }
        // 处理已拒绝专家与重复抽取
        BizUtils.notEmpty(expertGroupByStatus.get(REFUSED), refuseExperts -> {
            List<Long> tmpExpertIdsNotIn;
            if (reInvite) {
                tmpExpertIdsNotIn = refuseExperts.stream()
                        .filter(w -> w.getCreateOn().isAfter(tsTime))
                        .map(MeetingExpert::getExpertId)
                        .collect(Collectors.toList());
            } else {
                tmpExpertIdsNotIn = CollUtils.fieldList(refuseExperts, MeetingExpert::getExpertId);
            }
            expertIdsNotIn.addAll(tmpExpertIdsNotIn);
        });

        boolean hasExpertIdIn = !expertIdsIn.isEmpty();
        boolean hasExpertIdNotIn = !expertIdsNotIn.isEmpty();
        if (hasExpertIdIn && hasExpertIdNotIn) {
            expertIdsIn.removeAll(expertIdsNotIn);
            if (expertIdsIn.isEmpty()) {
                return result;
            }
            query.in(ExpertUserFullInfo::getUserId, expertIdsIn);
        } else if (hasExpertIdIn) {
            query.in(ExpertUserFullInfo::getUserId, expertIdsIn);
        } else if (hasExpertIdNotIn) {
            query.notIn(ExpertUserFullInfo::getUserId, expertIdsNotIn);
        }
        query.notIn(!avoidUnitCodes.isEmpty(), ExpertUserFullInfo::getCompanyUniqCode, avoidUnitCodes);
        List<ExpertUserFullInfo> userFullInfos = expertUserFullInfoService.list(query);
        if (userFullInfos.isEmpty()) {
            return result;
        }
        switch (avoidType) {
            case NONE:
                result.setExperts(inviteRandom(userFullInfos, expertsByRecentMeeting(), count));
                result.setTotal(userFullInfos.size());
                break;
            case CURR_UNIT:
                Map<String, List<ExpertUserFullInfo>> unitGroup = CollUtils.group(userFullInfos, ExpertUserFullInfo::getCompanyUniqCode);
                result.setExperts(inviteGroupByUnit(unitGroup, expertsByRecentMeeting(), count));
                result.setTotal(unitGroup.size());
                break;
            case CURR_STRIP:
                List<Long> userIds = CollUtils.fieldList(userFullInfos, ExpertUserFullInfo::getUserId);
                List<ExpertGovBusinessStrip> expertStrips = expertGovBusinessStripService.listByUserIds(userIds);
                if (!expertStrips.isEmpty()) {
                    Map<String, List<ExpertGovBusinessStrip>> stripGroup = CollUtils.group(expertStrips, ExpertGovBusinessStrip::getBusinessStripCode);
                    result.setExperts(inviteGroupByStrip(stripGroup, userFullInfos, expertsByRecentMeeting(), count));
                    result.setTotal(stripGroup.size());
                } else {
                    result.setTotal(0);
                }
                break;
        }
        return result;
    }

    private List<List<MeetingExpert>> expertsByRecentMeeting() {
        LambdaQueryWrapper<Meeting> query = Wrappers.lambdaQuery(Meeting.class)
                .select(Meeting::getId)
                .eq(Meeting::getInviteStatus, Boolean.TRUE)
                .ne(Meeting::getStatus, MeetingStatusEnum.CANCEL.getCode())
                .orderByDesc(Meeting::getCreateOn)
                .last("limit " + recentMeetingCount);
        List<Long> meetingIds = CollUtils.fieldList(meetingService.list(query), Meeting::getId);
        if (meetingIds.isEmpty()) {
            return Collections.emptyList();
        }
        Map<Long, List<MeetingExpert>> groupByMeetingId = meetingExpertService.listByMeetingIds(meetingIds)
                .stream().collect(Collectors.groupingBy(MeetingExpert::getMeetingId));
        return groupByMeetingId.entrySet().stream()
                .sorted(Map.Entry.comparingByKey(Comparator.reverseOrder()))
                .map(Map.Entry::getValue)
                .collect(Collectors.toList());
    }

    /**
     * 专家抽取（会议创建时抽取）
     *
     * @param randomRules 随机抽取规则
     * @param avoidRule   回避信息
     * @param meeting     会议信息
     * @author WendyYang
     **/
    public void expertInviteByMeetingCreate(Meeting meeting,
                                            List<RandomInviteRuleDTO> randomRules,
                                            AvoidRuleDTO avoidRule) {
        List<MeetingExpert> expertInserts = new ArrayList<>();
        // 处理随机抽取规则
        if (CollUtil.isNotEmpty(randomRules)) {
            List<ExpertInviteRule> randoms = new ArrayList<>();
            List<ExpertChoseDTO> expertsByRandom = new ArrayList<>();
            List<Long> chooseExpertIds = new ArrayList<>();
            LocalDateTime sTime = meeting.getStartTime();
            LocalDateTime eTime = meeting.getEndTime();
            LocalDateTime createOn = meeting.getCreateOn();
            randomRules.forEach(rule -> {
                ExpertChoseDTO tmpExperts = expertInviteByRandomRule(avoidRule, rule, chooseExpertIds, sTime, eTime, createOn);
                if (tmpExperts.getTotal() < rule.getCount()) {
                    throw ReturnException.wrap("可抽取专家数量不足");
                }
                expertsByRandom.add(tmpExperts);
                chooseExpertIds.addAll(CollUtils.fieldList(tmpExperts.getExperts(), ExpertUserFullInfo::getUserId));
                randoms.add(getExpertInviteRule(rule, meeting.getId()));
            });
            inviteRuleService.saveBatch(randoms);
            for (int i = 0; i < expertsByRandom.size(); i++) {
                Long ruleId = randoms.get(i).getId();
                expertsByRandom.get(i).getExperts().forEach(w -> {
                    MeetingExpert expert = ExpertInviteBuilder.getExpertByRandom(meeting.getId(), w, ruleId);
                    expert.setStatus(NOTICING.getCode());
                    expertInserts.add(expert);
                });
            }
            meetingNotifyHelper.notifyExperts(meeting, expertInserts);
        }
        meetingExpertService.saveBatch(expertInserts);
    }

}
