package com.hz.pm.api.meeting.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hz.pm.api.meeting.entity.domain.ExpertInviteRule;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.entity.dto.CountConfirmByMeetingIdDTO;
import com.hz.pm.api.meeting.entity.dto.ExpertInvitedRecordDTO;
import com.hz.pm.api.meeting.entity.dto.ReviewProjectDTO;
import com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.hz.pm.api.meeting.entity.enumeration.ExpertInviteTypeEnum;
import com.hz.pm.api.meeting.entity.req.ReviewProjectListReq;
import com.hz.pm.api.meeting.mapper.ExpertInviteRuleMapper;
import com.hz.pm.api.meeting.mapper.MeetingExpertMapper;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * 事务专家表 服务实现类
 * </p>
 *
 * @author WendyYang
 * @since 2022-07-27
 */
@Service
@RequiredArgsConstructor
public class MeetingExpertServiceImpl extends ServiceImpl<MeetingExpertMapper, MeetingExpert> implements IMeetingExpertService {

    private final ExpertInviteRuleMapper inviteRuleMapper;

    @Override
    public List<ExpertInvitedRecordDTO> listByExpertIdAndStatus(Long expertId, Integer status, List<Long> meetingIds) {
        return baseMapper.selectByExpertIdAndStatus(expertId, status, meetingIds);
    }

    @Override
    public List<ExpertInvitedRecordDTO> listByExpertIdAndTime(Long expertId, LocalDateTime sTime, LocalDateTime eTime) {
        return baseMapper.selectByExpertIdAndTime(expertId, sTime, eTime);
    }

    @Override
    public Map<Long, CountConfirmByMeetingIdDTO> countConfirmedByMeetingIds(List<Long> meetingIds) {
        List<ExpertInviteRule> inviteRules = inviteRuleMapper.selectList(Wrappers.lambdaQuery(ExpertInviteRule.class)
                .eq(ExpertInviteRule::getInviteType, ExpertInviteTypeEnum.RANDOM.getCode())
                .in(ExpertInviteRule::getMeetingId, meetingIds));
        Map<Long, Integer> countByRandom = CollUtils.groupSumInt(inviteRules, ExpertInviteRule::getMeetingId, ExpertInviteRule::getInviteCount);
        Page<MeetingExpert> page = pageExpertByStatusAndMeetingIds(new Page<>(0, 500), meetingIds, null);
        return page.getRecords().stream().collect(Collectors.groupingBy(MeetingExpert::getMeetingId,
                Collectors.collectingAndThen(Collectors.toList(),
                        w -> {
                            Long meetingId = w.get(0).getMeetingId();
                            CountConfirmByMeetingIdDTO confirm = new CountConfirmByMeetingIdDTO();
                            confirm.setConfirmed(0);
                            confirm.setTotal(countByRandom.getOrDefault(meetingId, 0));
                            confirm.setMeetingId(meetingId);

                            w.forEach(item -> {
                                ExpertAttendStatusEnum attendStatus = ExpertAttendStatusEnum.getByCode(item.getStatus());
                                if (ExpertInviteTypeEnum.APPOINT.eq(item.getInviteType())) {
                                    confirm.setTotal(confirm.getTotal() + 1);
                                }
                                // 除通知中的均为已确认
                                if (attendStatus.equals(ExpertAttendStatusEnum.AGREED)) {
                                    confirm.setConfirmed(confirm.getConfirmed() + 1);
                                }
                            });
                            return confirm;
                        })));
    }

    @Override
    public List<MeetingExpert> listByMeetingId(Long meetingId) {
        LambdaQueryWrapper<MeetingExpert> query = Wrappers.lambdaQuery(MeetingExpert.class)
                .eq(MeetingExpert::getMeetingId, meetingId)
                .orderByDesc(MeetingExpert::getUpdateOn);
        return list(query);
    }

    @Override
    public List<MeetingExpert> listByMeetingIds(List<Long> meetingIds) {
        LambdaQueryWrapper<MeetingExpert> query = Wrappers.lambdaQuery(MeetingExpert.class)
                .in(MeetingExpert::getMeetingId, meetingIds)
                .orderByDesc(MeetingExpert::getUpdateOn);
        return list(query);
    }

    @Override
    public MeetingExpert getByMeetingIdAndExpertId(Long meetingId, Long expertId) {
        LambdaQueryWrapper<MeetingExpert> query = Wrappers.lambdaQuery(MeetingExpert.class)
                .eq(MeetingExpert::getExpertId, expertId)
                .eq(MeetingExpert::getMeetingId, meetingId)
                .orderByDesc(MeetingExpert::getUpdateOn)
                .last("limit 1");
        return getOne(query);
    }

    @Override
    public Page<MeetingExpert> pageExpertByStatusAndMeetingId(Page<MeetingExpert> page, Long meetingId, ExpertAttendStatusEnum status, Integer inviteType) {
        Integer statusCode = Optional.ofNullable(status).flatMap(w -> Optional.of(w.getCode())).orElse(null);
        return baseMapper.selectExpertByStatusAndMeetingId(page, statusCode, meetingId, inviteType);
    }

    @Override
    public Page<MeetingExpert> pageExpertByStatusAndMeetingIds(Page<MeetingExpert> page, List<Long> meetingIds, ExpertAttendStatusEnum status) {
        return baseMapper.selectExpertByStatusAndMeetingIds(page, status == null ? null : status.getCode(), meetingIds);
    }

    @Override
    public int countExpertByStatusAndMeetingId(ExpertAttendStatusEnum status, Long meetingId, ExpertInviteTypeEnum inviteType) {
        Integer tempStatus = status == null ? null : status.getCode();
        Integer tempInviteType = inviteType == null ? null : inviteType.getCode();
        return baseMapper.countExpertByStatusAndMeetingId(tempStatus, meetingId, tempInviteType);
    }

    @Override
    public List<MeetingExpert> listAgreedExperts(Collection<Long> meetingIds) {
        LambdaQueryWrapper<MeetingExpert> query = Wrappers.lambdaQuery(MeetingExpert.class)
                .eq(MeetingExpert::getStatus, ExpertAttendStatusEnum.AGREED.getCode())
                .in(MeetingExpert::getMeetingId, meetingIds)
                .orderByAsc(MeetingExpert::getCreateOn);
        return baseMapper.selectList(query);
    }

    @Override
    public List<MeetingExpert> listExpertLastByMeetingIds(Collection<Long> meetingIds) {
        return baseMapper.listExpertLastByMeetingIds(meetingIds);
    }

    @Override
    public Page<ReviewProjectDTO> pageReviewProjectList(ReviewProjectListReq req) {
        return baseMapper.pageReviewProjectList(req.page(), req);
    }

    @Override
    public List<Long> listAgreeExpertIdByRecentDaysAndAgreeCount(int agreeCount, LocalDateTime sTime, LocalDateTime eTime) {
        return baseMapper.listAgreeExpertIdByRecentDaysAndAgreeCount(agreeCount, sTime, eTime);
    }

}
