package com.hz.pm.api.meeting.task;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.util.ThreadPoolUtil;
import com.hz.pm.api.meeting.entity.domain.ExpertInviteRule;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.entity.dto.RandomInviteRuleDTO;
import com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.hz.pm.api.meeting.entity.enumeration.ExpertInviteTypeEnum;
import com.hz.pm.api.meeting.service.IExpertInviteRuleService;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.hz.pm.api.sys.model.entity.MsgCallRecord;
import com.hz.pm.api.sys.model.entity.MsgCallRecord.ReplyStatus;
import com.hz.pm.api.sys.service.IMsgCallRecordService;
import com.ningdatech.basic.util.CollUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;

import static com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum.NOTICING;

/**
 * <p>
 * ExpertCallResultRewriteTask
 * <br>
 * 专家电话结果回填
 * </p>
 *
 * @author WendyYang
 * @since 14:15 2022/8/18
 */
@Slf4j
@Component
@AllArgsConstructor
public class ExpertCallResultRewriteTask {

    private final RandomInviteProperties properties;
    private final IMeetingExpertService meetingExpertService;
    private final IExpertInviteRuleService inviteRuleService;
    private final IMsgCallRecordService msgCallRecordService;

    private static final int OVERTIME = 30;

    private static final String AGREE_KEY = "1";
    private static final String REFUSE_KEY = "2";

    @PostConstruct
    public void initTask() {
        if (Boolean.FALSE.equals(properties.getEnable())) {
            log.warn("随机邀请已关闭……");
            return;
        }
        // 处理电话结果回填
        Instant startTime = Instant.now().plus(properties.getResultRewriteFixedRate(), ChronoUnit.MINUTES);
        Duration fixedRate = Duration.ofMinutes(properties.getResultRewriteFixedRate());
        // 处理短信结果回填
        ThreadPoolUtil.SCHEDULER.scheduleAtFixedRate(this::expertReplyRewrite, startTime, fixedRate);
    }

    private void expertReplyRewrite() {
        log.info("开始执行短信回复查询任务");
        // 查询所有邀请的专家信息 状态为通话中的
        Wrapper<MeetingExpert> meQuery = Wrappers.lambdaQuery(MeetingExpert.class)
                .eq(MeetingExpert::getStatus, NOTICING.getCode())
                .isNotNull(MeetingExpert::getSubmitKey)
                .eq(MeetingExpert::getInviteType, ExpertInviteTypeEnum.RANDOM.getCode());
        List<MeetingExpert> experts = meetingExpertService.list(meQuery);
        if (experts.isEmpty()) {
            log.info("暂无短信回复任务执行");
            return;
        }
        List<String> submitKeys = CollUtils.fieldList(experts, MeetingExpert::getSubmitKey);
        List<String> mobiles = CollUtils.fieldList(experts, MeetingExpert::getMobile);
        Set<Long> ruleIds = CollUtils.fieldSet(experts, MeetingExpert::getRuleId);
        // 查询随机邀请回调等待时间
        Map<Long, Integer> callbackMinutes = new HashMap<>(ruleIds.size());
        List<ExpertInviteRule> rules = inviteRuleService.listByIds(ruleIds);
        for (ExpertInviteRule rule : rules) {
            RandomInviteRuleDTO ruleDetail = JSONUtil.toBean(rule.getInviteRule(), RandomInviteRuleDTO.class);
            callbackMinutes.put(rule.getId(), ruleDetail.getWaitForCallbackMinutes());
        }
        // 查询短信语音回执结果
        Wrapper<MsgCallRecord> mcrQuery = Wrappers.lambdaQuery(MsgCallRecord.class)
                .in(MsgCallRecord::getSubmitKey, submitKeys)
                .in(MsgCallRecord::getReceivePhone, mobiles)
                .isNotNull(MsgCallRecord::getReplyOn);
        List<MsgCallRecord> records = msgCallRecordService.list(mcrQuery);
        Map<String, MsgCallRecord> replayMap = CollUtils.listToMap(records,
                w -> w.getSubmitKey() + "#" + w.getReceivePhone());

        List<MeetingExpert> changes = new ArrayList<>();
        for (MeetingExpert expert : experts) {
            String submitKeyMobile = expert.getSubmitKey() + "#" + expert.getMobile();
            Integer overtime = callbackMinutes.getOrDefault(expert.getRuleId(), OVERTIME);
            MsgCallRecord mcr = replayMap.get(submitKeyMobile);
            buildStatus(mcr, overtime, expert).ifPresent(w -> {
                expert.setStatus(w.getCode());
                changes.add(expert);
            });
        }
        meetingExpertService.updateBatchById(changes);
    }

    /**
     * 构建专家状态
     *
     * @param mcr      回复记录
     * @param overtime 超时时间
     * @param expert   专家
     * @return 专家状态
     */
    private static Optional<ExpertAttendStatusEnum> buildStatus(MsgCallRecord mcr,
                                                                Integer overtime,
                                                                MeetingExpert expert) {
        LocalDateTime now = LocalDateTime.now();
        boolean isOvertime = expert.getCreateOn().plusMinutes(overtime).isBefore(now);
        ExpertAttendStatusEnum status;
        if (mcr != null || isOvertime) {
            if (isOvertime) {
                status = ExpertAttendStatusEnum.UNANSWERED;
            } else {
                ReplyStatus replyStatus = mcr.getReplyStatus();
                switch (replyStatus) {
                    case OVERTIME:
                    case FAILED:
                        status = ExpertAttendStatusEnum.UNANSWERED;
                        break;
                    case SUCCESS:
                        String reply = StrUtil.trim(mcr.getReplyContent());
                        if (AGREE_KEY.equals(reply)) {
                            status = ExpertAttendStatusEnum.AGREED;
                        } else {
                            status = ExpertAttendStatusEnum.REFUSED;
                        }
                        break;
                    default:
                        status = null;
                        break;
                }
            }
        } else {
            status = null;
        }
        return Optional.ofNullable(status);
    }

}
