package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.meeting.entity.dto.ProjectReviewResultDTO;
import com.hz.pm.api.projectdeclared.model.entity.ProjectReview;
import com.hz.pm.api.projectdeclared.model.req.ProjectReviewApplyReq;
import com.hz.pm.api.projectdeclared.model.vo.ReviewProgressStatisticsVO;
import com.hz.pm.api.projectdeclared.service.IProjectReviewService;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * ProjectReviewManage
 * </p>
 *
 * @author WendyYang
 * @since 10:54 2024/4/17
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectReviewManage {

    private final IProjectReviewService projectReviewService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final IProjectService projectService;
    private final ProcessInstanceService processInstanceService;
    private final IProjectInstService projectInstService;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final MhUnitCache mhUnitCache;
    private final UserInfoHelper userInfoHelper;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void projectReviewApply(ProjectReviewApplyReq req) {
        Project project = projectService.getById(req.getProjectId());
        if (!ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(project.getStatus())
                && !ProjectStatus.PROJECT_REVIEW_FAILED.eq(project.getStatus())) {
            throw BizException.wrap("该项目不允许申请项目评审");
        }

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        WflowModels model = processModelManage.getWflowModelsNoNull(ProjectProcessType.PROJECT_REVIEW,
                user.getMhUnitId());

        ProcessStartParamsVo processParam = new ProcessStartParamsVo();
        processParam.setUser(declaredProjectHelper.buildUser(user));
        processParam.setProcessUsers(Collections.emptyMap());
        processParam.setFormData(Collections.emptyMap());
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instanceId = processInstanceService.startProcessLs(model, processParam, orgMap);

        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instanceId);
        projectInst.setInstType(ProjectProcessType.PROJECT_REVIEW);
        projectInst.setProjectId(req.getProjectId());
        projectInstService.save(projectInst);
        projectStateMachineUtil.pass(project);
        projectService.updateById(project);
        ProjectReview projReview = projectReviewService.getOne(Wrappers
                .lambdaQuery(ProjectReview.class)
                .eq(ProjectReview::getProjectId, req.getProjectId())
                .orderByDesc(ProjectReview::getId)
                .last(BizConst.LIMIT_1));
        if (projReview == null) {
            projReview = new ProjectReview();
            projReview.setProjectCode(project.getProjectCode());
        }
        BeanUtil.copyProperties(req, projReview);
        projectReviewService.saveOrUpdate(projReview);
        log.info("项目评审提交成功 【{}】", instanceId);
    }

    public ProjectReviewResultDTO projectReviewDetail(Long projectId) {
        ProjectReview projReview = projectReviewService.getOne(Wrappers
                .lambdaQuery(ProjectReview.class)
                .eq(ProjectReview::getProjectId, projectId)
                .orderByDesc(ProjectReview::getId)
                .last(BizConst.LIMIT_1));
        return BeanUtil.copyProperties(projReview, ProjectReviewResultDTO.class);
    }

    /**
     * 项目列表
     *
     * @param req \
     * @return \
     */
    public ReviewProgressStatisticsVO reviewProjectProgressStatistics(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return null;
        }
        req.setStatus(null);
        ProjectManageUtil.projectQuery(query, req);
        query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .eq(Project::getNewest, Boolean.TRUE)
                .select(Project::getStatus, Project::getId);
        List<Project> projects = projectService.list(query);
        Map<ProjectStatus, Long> countMap = CollUtils.groupCount(projects, w -> {
            if (!ProjectStatus.PROJECT_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.COMPLIANCE_REVIEW_PASSED.eq(w.getStatus())
                    && !ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_CHANGE_APPLY.eq(w.getStatus())) {
                return ProjectStatus.PROJECT_REVIEW_PASSED;
            }
            return ProjectStatus.getNoNull(w.getStatus());
        });
        return ReviewProgressStatisticsVO.builder()
                .totalCount(projects.size())
                .todoCount(countMap.getOrDefault(ProjectStatus.WITHOUT_PROJECT_REVIEW, 0L))
                .auditCount(countMap.getOrDefault(ProjectStatus.ON_PROJECT_REVIEW, 0L))
                .passedCount(countMap.getOrDefault(ProjectStatus.PROJECT_REVIEW_PASSED, 0L))
                .failedCount(countMap.getOrDefault(ProjectStatus.PROJECT_REVIEW_FAILED, 0L))
                .build();
    }

    /**
     * 项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> pageReviewProject(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        if (!projectQueryPermission(query, user, req.getDeclaredUnitId())) {
            return PageVo.empty();
        }
        Integer status = req.getStatus();
        if (status != null) {
            req.setStatus(null);
            if (ProjectStatus.PROJECT_REVIEW_PASSED.eq(status)) {
                query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                        " and npsc.event = {0}", ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
            } else {
                query.eq(Project::getStatus, status);
            }
        } else {
            query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                    " and npsc.event = {0}", ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS);
        }
        ProjectManageUtil.projectQuery(query, req);
        query.eq(Project::getNewest, Boolean.TRUE)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode());
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = BeanUtil.copyProperties(w, ProjectLibListItemVO.class);
            if (!ProjectStatus.PROJECT_REVIEW_FAILED.eq(w.getStatus())
                    && !ProjectStatus.COMPLIANCE_REVIEW_PASSED.eq(w.getStatus())
                    && !ProjectStatus.WITHOUT_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_PROJECT_REVIEW.eq(w.getStatus())
                    && !ProjectStatus.ON_CHANGE_APPLY.eq(w.getStatus())) {
                item.setStatus(ProjectStatus.PROJECT_REVIEW_PASSED.getCode());
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    private boolean projectQueryPermission(LambdaQueryWrapper<Project> query,
                                           UserFullInfoDTO user,
                                           Long declaredUnitId) {
        boolean queryState = true;
        Optional<DataScopeDTO> currDs = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!currDs.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            switch (currDs.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER: {
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    if (declaredUnitId != null && childUnitIds.contains(declaredUnitId)
                            && !Objects.equals(declaredUnitId, user.getMhUnitId())) {
                        childUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                    }
                    List<String> viewUnitIdList = CollUtils.convert(childUnitIds, String::valueOf);
                    //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    query.and(q1 -> q1.in(Project::getBuildOrgCode, viewUnitIdList)
                            .or(q2 -> q2.eq(Project::getStage, ProjectStatus.NOT_APPROVED.getCode())
                                    .in(Project::getSuperOrgCode, viewUnitIdList)));
                }
                break;
                case SUPER_ADMIN: {
                    if (declaredUnitId != null) {
                        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                        List<String> viewUnitIdList = CollUtils.convert(viewUnitIds, String::valueOf);
                        query.and(q1 -> q1.in(Project::getBuildOrgCode, viewUnitIdList)
                                .or(q2 -> q2.eq(Project::getStage, ProjectStatus.NOT_APPROVED.getCode())
                                        .in(Project::getSuperOrgCode, viewUnitIdList)));
                    }
                }
                break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
        }
        return queryState;
    }

}
