package com.hz.pm.api.projectdeclared.utils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;

/**
 * <p>
 * DateFormatUtil
 * </p>
 *
 * @author WendyYang
 * @since 12:59 2024/8/9
 */
public class DateFormatUtil {

    private DateFormatUtil() {
    }

    /**
     * 判断对象是否为日期类型，并进行格式化。
     *
     * @param obj 要判断的对象
     * @return 如果是日期类型则返回格式化的字符串，否则返回 null
     */
    public static String format(Object obj) {
        // 检查对象是否为日期类型
        if (isDateType(obj)) {
            // 根据日期类型选择合适的 DateTimeFormatter
            DateTimeFormatter formatter = getDateTimeFormatter(obj);
            if (formatter != null) {
                // 格式化日期
                return format(obj, formatter);
            }
        }
        return null;
    }

    /**
     * 判断对象是否为日期类型。
     *
     * @param obj 要判断的对象
     * @return 如果是日期类型返回 true，否则返回 false
     */
    public static boolean isDateType(Object obj) {
        if (obj != null) {
            Class<?> clazz = obj.getClass();
            return clazz.isAssignableFrom(LocalDate.class)
                    || clazz.isAssignableFrom(LocalDateTime.class)
                    || clazz.isAssignableFrom(LocalTime.class)
                    || clazz.isAssignableFrom(Date.class);
        }
        return false;
    }

    /**
     * 根据日期类型获取对应的 DateTimeFormatter。
     *
     * @param obj 日期对象
     * @return 对应的 DateTimeFormatter
     */
    private static DateTimeFormatter getDateTimeFormatter(Object obj) {
        if (obj instanceof LocalDate) {
            return DateTimeFormatter.ofPattern("yyyy-MM-dd");
        } else if (obj instanceof LocalDateTime) {
            return DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        } else if (obj instanceof LocalTime) {
            return DateTimeFormatter.ofPattern("HH:mm:ss");
        } else if (obj instanceof Date) {
            return DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        }
        return null;
    }

    /**
     * 格式化日期对象为字符串。
     *
     * @param obj       日期对象
     * @param formatter 日期格式器
     * @return 格式化的字符串
     */
    private static String format(Object obj, DateTimeFormatter formatter) {
        if (obj instanceof LocalDate) {
            return ((LocalDate) obj).format(formatter);
        } else if (obj instanceof LocalDateTime) {
            return ((LocalDateTime) obj).format(formatter);
        } else if (obj instanceof LocalTime) {
            return ((LocalTime) obj).format(formatter);
        } else if (obj instanceof Date) {
            return LocalDateTime.ofInstant(((Date) obj).toInstant(), ZoneId.systemDefault()).format(formatter);
        }
        return null;
    }

}
