package com.hz.pm.api.projectdeclared.utils;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ClassUtil;
import cn.hutool.core.util.ObjUtil;
import lombok.Builder;
import lombok.Data;

import java.util.*;

/**
 * <p>
 * BeanDiffUtil
 * </p>
 *
 * @author WendyYang
 * @since 10:27 2024/8/6
 */
public final class ProjectDetailDiffUtil {

    private ProjectDetailDiffUtil() {
    }

    @Data
    @Builder
    public static class FieldDiff {

        private String fieldName;

        private String fieldType;

        private Object oldValue;

        private Object newValue;

    }

    /**
     * 比较两个对象的不同属性值，包括嵌套的对象属性。
     *
     * @param oldObj   oldObj
     * @param newObj   newObj
     * @param diffSame 是否只对比相同属性
     * @param ignores  忽略比较的字段
     * @return 对比结果
     **/
    public static List<FieldDiff> diff(Object oldObj, Object newObj, boolean diffSame, String... ignores) {
        if (oldObj == null && newObj == null) {
            return Collections.emptyList();
        }
        Map<String, Object> oldObjMap = oldObj == null ? Collections.emptyMap() : BeanUtil.beanToMap(oldObj);
        Map<String, Object> newObjMap = newObj == null ? Collections.emptyMap() : BeanUtil.beanToMap(newObj);
        if (ArrayUtil.isNotEmpty(ignores)) {
            for (String ignore : ignores) {
                newObjMap.remove(ignore);
                oldObjMap.remove(ignore);
            }
        }
        // 创建结果 Map
        List<FieldDiff> result = new ArrayList<>();

        // 获取两个 Map 的 keySet
        Set<String> oldObjKeys = oldObjMap.keySet();
        Set<String> newObjKeys = newObjMap.keySet();

        // 计算不同的属性
        Collection<String> diffKeys = CollUtil.disjunction(oldObjKeys, newObjKeys);

        if (!diffSame) {
            // 输出不同的属性及其值
            for (String key : diffKeys) {
                Object oldVal = oldObjMap.get(key);
                Object newVal = newObjMap.get(key);
                result.add(FieldDiff.builder()
                        .fieldName(key)
                        .fieldType((oldObj == null ? newVal.getClass() : oldVal.getClass()).getTypeName())
                        .oldValue(formatDate(oldVal))
                        .newValue(formatDate(newVal))
                        .build());
            }

        }

        // 输出相同的属性，但是值不同
        Collection<String> sameKeys = CollUtil.intersection(oldObjKeys, newObjKeys);
        for (String key : sameKeys) {
            Object oldVal = oldObjMap.get(key);
            Object newVal = newObjMap.get(key);
            if (oldVal == null && newVal == null) {
                continue;
            }
            Class<?> clazz = oldVal == null ? newVal.getClass() : oldVal.getClass();
            oldVal = formatDate(MapUtil.get(oldObjMap, key, clazz));
            newVal = formatDate(MapUtil.get(newObjMap, key, clazz));
            if (ClassUtil.isSimpleValueType(clazz) && ObjUtil.notEqual(oldVal, newVal)) {
                result.add(FieldDiff.builder()
                        .fieldName(key)
                        .fieldType(clazz.getTypeName())
                        .oldValue(oldVal)
                        .newValue(newVal)
                        .build());
            }
        }
        return result;
    }

    private static Object formatDate(Object date) {
        if (DateFormatUtil.isDateType(date)) {
            return DateFormatUtil.format(date);
        }
        return date;
    }

}
