package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessInstanceVo;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.ProcessStatusEnum;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.springframework.core.Ordered;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 流程业务抽象处理类
 *
 * @author CMM
 * @since 2023/02/24 14:22
 */

public abstract class AbstractProcessHandle implements Ordered, ProcessHandle {
    /**
     * 构造，子类必须实现
     */
    protected AbstractProcessHandle() {

    }

    @Getter
    @AllArgsConstructor
    public enum ProcessNode {

        PROJECT_DECLARED("项目申报", 1),
        PROJECT_REVIEW("项目评审", 2),
        APPROVAL_AMOUNT("资金下达", 3),
        PROJECT_APPROVAL("项目立项", 4),
        PROJECT_PURCHASE("招标采购", 5),
        PROJECT_ADAPTION("上传开工文件", 6),
        SYSTEM_SELF_TEST("系统自测", 7),
        SYSTEM_TEST_VALID("监理核实验证", 8),
        FIRST_INSPECTED("项目初验", 9),
        PILOT_RUNNING("上线试运行", 10),
        XCFHX_REVIEW("信创测评", 11),
        FINAL_INSPECTED("项目终验", 12);


        private final String name;
        private final Integer order;

        public static final List<ProcessNode> ALL = Arrays.stream(ProcessNode.values())
                .sorted(Comparator.comparingInt(ProcessNode::getOrder))
                .collect(Collectors.toList());

    }

    private static final Map<ProcessStatusEnum, Integer> PROCESS_STATUS_PRIORITY_MAP;

    static {
        PROCESS_STATUS_PRIORITY_MAP = new EnumMap<>(ProcessStatusEnum.class);
        PROCESS_STATUS_PRIORITY_MAP.put(ProcessStatusEnum.APPROVED, 3);
        PROCESS_STATUS_PRIORITY_MAP.put(ProcessStatusEnum.UNDER_REVIEW, 1);
        PROCESS_STATUS_PRIORITY_MAP.put(ProcessStatusEnum.BE_REJECTED, 2);
    }

    protected Map<Long, PurchaseInst> getPurchaseInstMap(List<PurchaseInst> instList,
                                                         Collection<Long> bidIds,
                                                         InstTypeEnum instType) {
        return instList.stream()
                .filter(w -> instType.eq(w.getInstType()) && bidIds.contains(w.getBidId()))
                .collect(Collectors.toMap(PurchaseInst::getBidId, Function.identity(),
                        (w1, w2) -> w1.getCreateOn().isAfter(w2.getCreateOn()) ? w1 : w2));
    }

    protected static StepStatusEnum getPurchaseStatus(List<ProcessInfoVO> processes) {
        Optional<Integer> min = processes.stream().map(w -> {
            ProcessStatusEnum status = Arrays.stream(ProcessStatusEnum.values())
                    .filter(s -> s.getDesc().equals(w.getProcessProgressVo().getStatus()))
                    .findFirst()
                    .orElse(ProcessStatusEnum.NOT_SUBMITTED);
            return PROCESS_STATUS_PRIORITY_MAP.getOrDefault(status, 0);
        }).min(Comparator.comparingInt(Integer::intValue));
        if (min.isPresent()) {
            int miniOrder = min.get();
            if (miniOrder == 1) {
                return StepStatusEnum.ON_GOING;
            } else if (miniOrder == 2) {
                return StepStatusEnum.REJECTED;
            } else {
                return StepStatusEnum.COMPLETED;
            }
        } else {
            return StepStatusEnum.NOT_START;
        }
    }

    public void buildStepStatus(ProcessDetailVO detail, ProcessProgressVo progress, ProjectStatusChange event) {
        detail.setProcessProgressVo(progress);
        if (ProcessStatusEnum.BE_REJECTED.getDesc().equals(progress.getStatus())) {
            detail.setStepStatus(StepStatusEnum.REJECTED);
            detail.setFinishTime(event.getCreateOn());
        } else if (ProcessStatusEnum.APPROVED.getDesc().equals(progress.getStatus())) {
            detail.setStepStatus(StepStatusEnum.COMPLETED);
            detail.setFinishTime(event.getCreateOn());
        } else {
            detail.setStepStatus(StepStatusEnum.ON_GOING);
        }
    }

}
