package com.hz.pm.api.projectlib.helper;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.model.constant.MhUnitConst;
import com.hz.pm.api.common.statemachine.event.AbstractStateChangeEvent;
import com.hz.pm.api.performance.model.entity.ProjectCoreBusinessIndicators;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectApplication;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectApplicationVO;
import com.hz.pm.api.user.helper.MhUnitCache;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * ProjectHelperUtil
 * </p>
 *
 * @author WendyYang
 * @since 14:15 2023/2/11
 */
public class ProjectManageUtil {

    private static volatile MhUnitCache unitCache;

    private static MhUnitCache getMhUnitCache() {
        if (unitCache == null) {
            synchronized (ProjectManageUtil.class) {
                if (unitCache == null) {
                    unitCache = SpringUtil.getBean(MhUnitCache.class);
                }
            }
        }
        return unitCache;
    }

    private ProjectManageUtil() {

    }

    public static LambdaQueryWrapper<Project> initQuery() {
        return new LambdaQueryWrapper<>();
    }

    /**
     * 通用项目查询
     *
     * @param req 查询条件
     * @return /
     **/
    public static void projectQuery(LambdaQueryWrapper<Project> query, ProjectListReq req) {
        query.eq(req.getUserId() != null, Project::getCreateBy, req.getUserId())
                .like(StrUtil.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .eq(req.getProjectType() != null, Project::getProjectType, req.getProjectType())
                .eq(req.getProjectYear() != null, Project::getProjectYear, req.getProjectYear())
                .eq(req.getStage() != null, Project::getStage, req.getStage())
                .eq(req.getStatus() != null, Project::getStatus, req.getStatus())
                // 申报金额
                .ge(req.getDeclareAmountMin() != null, Project::getDeclareAmount, req.getDeclareAmountMin())
                .le(req.getDeclareAmountMax() != null, Project::getDeclareAmount, req.getDeclareAmountMax())
                // 申报时间
                .ge(req.getCreateOnMin() != null, Project::getCreateOn, req.getCreateOnMin())
                .le(req.getCreateOnMax() != null, Project::getCreateOn, req.getCreateOnMax())
                // 批复金额
                .ge(req.getApproveAmountMin() != null, Project::getApprovalAmount, req.getApproveAmountMin())
                .le(req.getApproveAmountMax() != null, Project::getApprovalAmount, req.getApproveAmountMax())

                .like(StrUtil.isNotBlank(req.getBuildOrg()), Project::getBuildOrgName, req.getBuildOrg())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .like(StrUtil.isNotBlank(req.getBuildOrgName()), Project::getBuildOrgName, req.getBuildOrgName())
                .eq(req.getSuperOrgCode() != null, Project::getSuperOrgCode, req.getSuperOrgCode())
                .eq(req.getIsTemporaryAugment() != null, Project::getIsTemporaryAugment, req.getIsTemporaryAugment())
                //状态 阶段 list
                .in(CollUtil.isNotEmpty(req.getStageList()), Project::getStage, req.getStageList())
                .in(CollUtil.isNotEmpty(req.getStatusList()), Project::getStatus, req.getStatusList())
                //实例code
                .in(CollUtil.isNotEmpty(req.getInstCodes()), Project::getInstCode, req.getInstCodes())
                //已添加评价计划的项目code
                .notIn(CollUtil.isNotEmpty(req.getProjectCodes()), Project::getProjectCode, req.getProjectCodes())
                .eq(req.getIsStartDeclaredProject() != null, Project::getAnnualPlanOpened, req.getIsStartDeclaredProject())
                .orderByDesc(Project::getUpdateOn);
        // 处理申报单位查询
        if (req.getBuildOrgCode() != null && NumberUtil.isLong(req.getBuildOrgCode())) {
            long buildOrgCode = Long.parseLong(req.getBuildOrgCode());
            if (buildOrgCode != MhUnitConst.ROOT_UNIT_ID) {
                List<Long> viewOrgCodes = getMhUnitCache().getViewChildIdsRecursion(buildOrgCode);
                query.in(Project::getBuildOrgCode, viewOrgCodes);
            }
        }
    }

    public static void projectBaseQuery(LambdaQueryWrapper<Project> query) {
        query.select(Project::getId, Project::getStage, Project::getStatus, Project::getProjectCode,
                Project::getBuildOrgCode, Project::getBuildOrgName, Project::getApprovalAmount, Project::getApprovalDate,
                Project::getCreateOn, Project::getProjectName, Project::getProjectYear, Project::getIsMajorProject,
                Project::getDeclareAmount, Project::getReviewAmount, Project::getUnitStrip);
    }

    /**
     * 通用项目查询
     *
     * @param req 查询条件
     * @return /
     **/
    public static LambdaQueryWrapper<Project> projectQuery(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = new LambdaQueryWrapper<>();
        projectQuery(query, req);
        return query;
    }

    public static void projectQueryByProjectStatusChange(LambdaQueryWrapper<Project> query,
                                                         AbstractStateChangeEvent event) {
        query.exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE +
                " and npsc.event = {0}", event);
    }

    public static void projectQueryByPurchaseStatusChange(LambdaQueryWrapper<Project> query,
                                                          AbstractStateChangeEvent event) {
        query.exists(ExistsSqlConst.PROJECT_EXISTS_PURCHASE_STATUS_CHANGE +
                " and npsc.event ={0}", event.name());
    }

    /**
     * 转换VO
     *
     * @param app
     * @return
     */
    public static ProjectApplicationVO to(ProjectApplication app,
                                          Map<Long, List<ProjectCoreBusinessIndicators>> coreBizMap) {
        ProjectApplicationVO appVo = new ProjectApplicationVO();
        BeanUtils.copyProperties(app, appVo);
        if (coreBizMap.containsKey(app.getId())) {
            List<ProjectCoreBusinessIndicators> projectCoreBusinessIndicators = coreBizMap.get(app.getId());
            appVo.setCoreBusinessList(projectCoreBusinessIndicators);
        }
        return appVo;
    }

    public static ProjectApplicationVO to(ProjectApplication app) {
        ProjectApplicationVO appVo = new ProjectApplicationVO();
        BeanUtils.copyProperties(app, appVo);
        return appVo;
    }
}
