package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.projectdeclared.model.entity.ProjectChangeHistory;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectHelper;
import com.hz.pm.api.projectdeclared.model.dto.*;
import com.hz.pm.api.projectdeclared.model.vo.*;
import com.hz.pm.api.projectdeclared.service.IProjectChangeHistoryService;
import com.hz.pm.api.projectdeclared.utils.ProjectDetailDiffUtil;
import com.hz.pm.api.projectdeclared.utils.ProjectDetailDiffUtil.FieldDiff;
import com.hz.pm.api.projectlib.helper.ProjectChangeStopHelper;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectChangeListDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectChangeStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.SubmitProjectChangeApplyReq;
import com.hz.pm.api.projectlib.model.vo.ProjectChangeDetailVO;
import com.hz.pm.api.projectlib.model.vo.ProjectChangeListVO;
import com.hz.pm.api.projectlib.model.vo.ProjectDetailVO;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT;

/**
 * <p>
 * ProjectStoppedManage
 * </p>
 *
 * @author WendyYang
 * @since 11:04 2024/8/5
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectChangeManage {

    private final IProjectService projectService;
    private final IProjectInstService projectInstService;
    private final ProcessInstanceService processInstanceService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final ProjectChangeStopHelper projectChangeStopHelper;
    private final ProjectLibManage projectLibManage;
    private final IProjectChangeHistoryService projectChangeHistoryService;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitProjectChangeApply(SubmitProjectChangeApplyReq req) {
        JSONObject projectEditDetail = req.getProjectEditDetail();
        JSONObject projJsonObj = projectEditDetail.getJSONObject(EditProjectDetailDTO.FIELD_PROJECT);
        ProjectDTO reqProj = projJsonObj.toBean(ProjectDTO.class);
        Project project = projectService.getNewProject(reqProj.getId());
        if (ProjectStatus.ON_CHANGE_APPLY.eq(project.getStatus()) ||
                ProjectStatus.STOPPED_PASSED.eq(project.getStatus())) {
            throw ReturnException.wrap("暂无法提交变更申请");
        }
        ProjectDetailVO projDetailOld = projectLibManage.getProjectDetail(project);
        List<FieldDiff> projDiffs = ProjectDetailDiffUtil.diff(projDetailOld, projJsonObj, true,
                EditProjectDTO.SYSTEM_REPLACE_INFOS, "id", "projectCode");
        int changeFieldsCount = projDiffs.size();
        // 系统替代信息变更比较
        List<GovSystemReplaceInfoDTO> systemReplaceInfos = reqProj.getSystemReplaceInfos();
        List<GovSystemReplaceInfoDTO> systemReplaceInfosOld = projDetailOld.getSystemReplaceInfos();
        if (!CollUtil.isEqualList(systemReplaceInfos, systemReplaceInfosOld)) {
            projDiffs.add(FieldDiff.builder()
                    .fieldName(EditProjectDTO.SYSTEM_REPLACE_INFOS)
                    .fieldType(List.class.getTypeName())
                    .oldValue(systemReplaceInfosOld)
                    .newValue(systemReplaceInfos)
                    .build());
            changeFieldsCount++;
        }
        projectEditDetail.set(EditProjectDetailDTO.FIELD_PROJECT_DIFFS, projDiffs);
        // 采购信息变更比较
        List<JSONObject> purchaseJsonObjs = projectEditDetail.getBeanList(EditProjectDetailDTO.FIELD_PURCHASES, JSONObject.class);
        if (CollUtil.isNotEmpty(purchaseJsonObjs)) {
            List<PurchaseFullInfoVO> purchaseDetails = projDetailOld.getPurchaseDetails();
            if (purchaseDetails != null && !purchaseDetails.isEmpty()) {
                Map<Long, JSONObject> reqPurchaseMap = CollUtils.listToMap(purchaseJsonObjs,
                        w -> w.getJSONObject(EditPurchaseDetailDTO.FIELD_PURCHASE).getLong(EditPurchaseDTO.FIELD_ID));
                for (PurchaseFullInfoVO purchaseDetail : purchaseDetails) {
                    PurchaseVO purchase = purchaseDetail.getPurchase();
                    ContractVO contract = purchaseDetail.getConstruction();
                    OperationVO operation = purchaseDetail.getOperation();
                    List<PreInsAcceptancePersonVO> firstInspectedPersons = purchaseDetail.getFirstInspectedPersons();

                    JSONObject reqPurchaseDetail = reqPurchaseMap.get(purchase.getId());
                    JSONObject reqPurchase = reqPurchaseDetail.getJSONObject(EditPurchaseDetailDTO.FIELD_PURCHASE);
                    JSONObject reqOperation = reqPurchaseDetail.getJSONObject(EditPurchaseDetailDTO.FIELD_OPERATION);
                    JSONObject reqContract = reqPurchaseDetail.getJSONObject(EditPurchaseDetailDTO.FIELD_CONTRACT);
                    List<PreInsAcceptancePersonVO> reqFirstInspectedPersons = reqPurchaseDetail.getBeanList(EditPurchaseDetailDTO.FIELD_FIRST_INSPECTED_PERSONS, PreInsAcceptancePersonVO.class);
                    // 初验人员变更比较
                    if (!CollUtil.isEqualList(firstInspectedPersons, reqFirstInspectedPersons)) {
                        changeFieldsCount++;
                        reqPurchaseDetail.set(EditPurchaseDetailDTO.FIELD_FIRST_INSPECTED_PERSONS_DIFF,
                                FieldDiff.builder()
                                        .fieldName(EditPurchaseDetailDTO.FIELD_FIRST_INSPECTED_PERSONS)
                                        .fieldType(List.class.getTypeName())
                                        .oldValue(firstInspectedPersons)
                                        .newValue(reqFirstInspectedPersons)
                                        .build());
                    }

                    List<FieldDiff> purchaseDiffs = ProjectDetailDiffUtil.diff(purchase, reqPurchase, true, "id");
                    List<FieldDiff> operationDiffs = ProjectDetailDiffUtil.diff(operation, reqOperation, true, "id");
                    List<FieldDiff> contractDiffs = ProjectDetailDiffUtil.diff(contract, reqContract, true, "id");
                    changeFieldsCount += purchaseDiffs.size() + operationDiffs.size() + contractDiffs.size();
                    reqPurchaseDetail.set(EditPurchaseDetailDTO.FIELD_PURCHASE_DIFFS, purchaseDiffs);
                    reqPurchaseDetail.set(EditPurchaseDetailDTO.FIELD_CONTRACT_DIFFS, contractDiffs);
                    reqPurchaseDetail.set(EditPurchaseDetailDTO.FIELD_OPERATION_DIFFS, operationDiffs);
                }
            }
        }
        if (changeFieldsCount == 0) {
            throw BizException.wrap("项目内容无修改");
        }
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();

        WflowModels wflowModels = processModelManage.getWflowModelsNoNull(ProjectProcessType.PROJECT_CHANGE, userDetail.getMhUnitId());

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(userDetail));
        params.setProcessUsers(Collections.emptyMap());
        params.setFormData(BeanUtil.beanToMap(req, false, true));

        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(userDetail, project);
        String instId = processInstanceService.startProcessLs(wflowModels, params, orgModelMap);
        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instId);
        projectInst.setProjectId(project.getId());
        projectInst.setInstType(ProjectProcessType.PROJECT_CHANGE);
        projectInstService.save(projectInst);
        projectChangeStopHelper.changeProjectPurchaseStatus(project, CHANGE_APPLY_SUBMIT);

        projectEditDetail.getConfig().setIgnoreNullValue(false);
        // 保存变更历史
        ProjectChangeHistory projectChangeHistory = new ProjectChangeHistory();
        projectChangeHistory.setChangeContent(projectEditDetail.toString());
        projectChangeHistory.setChangeReason(req.getChangeReason());
        projectChangeHistory.setProjectId(project.getId());
        projectChangeHistory.setProjectCode(project.getProjectCode());
        projectChangeHistory.setStatus(ProjectChangeStatus.ON_CHANGE_APPLY.getCode());
        projectChangeHistory.setInstCode(instId);
        projectChangeHistory.setId(req.getId());
        projectChangeHistoryService.saveOrUpdate(projectChangeHistory);
    }

    public PageVo<ProjectChangeListVO> page(ProjectListReq req) {
        LambdaQueryWrapper<Project> wrapper = ProjectManageUtil.projectQuery(req)
                .eq(Project::getNewest, Boolean.TRUE);
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        if (!userDetail.hasSuperAdmin()) {
            wrapper.eq(Project::getBuildOrgCode, userDetail.getMhUnitIdStr());
        }
        Page<ProjectChangeListDTO> page = projectService.pageChangeList(wrapper, req.page());
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectChangeListDTO> records = page.getRecords();
        List<ProjectChangeListVO> data = records.stream().map(w -> {
            ProjectChangeListVO item = new ProjectChangeListVO();
            BeanUtil.copyProperties(w, item);
            item.setUnitStripName(MhUnitStripEnum.getVal(w.getUnitStrip()));
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public ProjectChangeDetailVO changeDetail(Long id) {
        ProjectChangeHistory changeHistory = projectChangeHistoryService.getById(id);
        if (changeHistory == null) {
            return null;
        }
        ProjectChangeDetailVO result = new ProjectChangeDetailVO();
        result.setProjectEditDetail(JSONUtil.toBean(changeHistory.getChangeContent(), EditProjectDetailDTO.class));
        result.setChangeReason(changeHistory.getChangeReason());
        result.setCreateOn(changeHistory.getCreateOn());
        return result;
    }

    public ProjectChangeDetailVO changeDetail(Long projectId, String instCode) {
        Wrapper<ProjectChangeHistory> query = Wrappers.lambdaQuery(ProjectChangeHistory.class)
                .eq(ProjectChangeHistory::getProjectId, projectId)
                .eq(ProjectChangeHistory::getInstCode, instCode);
        ProjectChangeHistory changeHistory = projectChangeHistoryService.getOne(query);
        if (changeHistory == null) {
            return null;
        }
        ProjectChangeDetailVO result = new ProjectChangeDetailVO();
        result.setProjectEditDetail(JSONUtil.toBean(changeHistory.getChangeContent(), EditProjectDetailDTO.class));
        result.setChangeReason(changeHistory.getChangeReason());
        result.setCreateOn(changeHistory.getCreateOn());
        return result;
    }

    public List<ProjectLibListItemVO> options() {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode, Project::getProjectName)
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getCreateOn)
                .notExists("select 1 from nd_project_change_history npch where " +
                                "npch.project_code = nd_project.project_code and npch.status = {0}",
                        ProjectChangeStatus.CHANGE_FAILED.getCode())
                .notIn(Project::getStatus, ProjectStatus.STOPPED_PASSED.getCode(),
                        ProjectStatus.ON_STOPPED_APPLY.getCode(),
                        ProjectStatus.ON_CHANGE_APPLY.getCode());
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        if (!userDetail.hasSuperAdmin()) {
            query.eq(Project::getBuildOrgCode, userDetail.getMhUnitIdStr());
        }
        List<Project> projects = projectService.list(query);
        return CollUtils.convert(projects, w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setId(w.getId());
            return item;
        });
    }

}
