package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectHelper;
import com.hz.pm.api.projectlib.helper.ProjectChangeStopHelper;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.SubmitProjectStoppedApplyReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.ProjectStoppedDetailVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT;

/**
 * <p>
 * ProjectStoppedManage
 * </p>
 *
 * @author WendyYang
 * @since 11:04 2024/8/5
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectStoppedManage {

    private final IProjectService projectService;
    private final IProjectInstService projectInstService;
    private final ProcessInstanceService processInstanceService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final ProjectChangeStopHelper projectChangeStopHelper;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitProjectStoppedApply(SubmitProjectStoppedApplyReq req) {
        Project project = projectService.getNewProject(req.getProjectId());
        if (ProjectStatus.STOPPED_PASSED.eq(project.getStatus()) ||
                ProjectStatus.ON_STOPPED_APPLY.eq(project.getStatus())) {
            throw ReturnException.wrap("暂无法提交终止申请");
        }
        project.setStoppedFiles(req.getStoppedFiles());
        project.setStoppedReason(req.getStoppedReason());
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();

        WflowModels wflowModels = processModelManage.getWflowModelsNoNull(ProjectProcessType.PROJECT_STOPPED, userDetail.getMhUnitId());

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(userDetail));
        params.setProcessUsers(Collections.emptyMap());
        params.setFormData(BeanUtil.beanToMap(req, false, true));

        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(userDetail, project);
        String instId = processInstanceService.startProcessLs(wflowModels, params, orgModelMap);
        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instId);
        projectInst.setProjectId(project.getId());
        projectInst.setInstType(ProjectProcessType.PROJECT_STOPPED);
        projectInstService.save(projectInst);
        // 修改项目状态、标项状态
        projectChangeStopHelper.changeProjectPurchaseStatus(project, STOPPED_APPLY_SUBMIT);
    }

    public PageVo<ProjectLibListItemVO> page(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        query.eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getUpdateOn)
                .isNotNull(Project::getStoppedStatus);
        Page<Project> page = req.page();
        projectService.page(page, query);
        List<Project> records = page.getRecords();
        if (CollUtil.isEmpty(records)) {
            return PageVo.empty();
        }
        Wrapper<ProjectInst> projectInstQuery = Wrappers.lambdaQuery(ProjectInst.class)
                .eq(ProjectInst::getInstType, ProjectProcessType.PROJECT_STOPPED.getCode())
                .in(ProjectInst::getProjectId, CollUtils.fieldList(records, Project::getId));
        List<ProjectInst> projectInstList = projectInstService.list(projectInstQuery);
        Map<Long, ProjectInst> projectInstMap = BizUtils.groupFirstMap(projectInstList,
                ProjectInst::getProjectId,
                Comparator.comparing(ProjectInst::getCreateOn).reversed());
        List<ProjectLibListItemVO> data = records.stream().map(w -> {
            ProjectLibListItemVO project = new ProjectLibListItemVO();
            project.setProjectName(w.getProjectName());
            project.setProjectCode(w.getProjectCode());
            project.setStoppedStatus(w.getStoppedStatus());
            project.setId(w.getId());
            ProjectInst projectInst = projectInstMap.get(w.getId());
            if (projectInst != null) {
                project.setInstCode(projectInst.getInstCode());
            }
            project.setStage(w.getStage());
            project.setStatus(w.getStatus());
            project.setCreateOn(w.getCreateOn());
            project.setBuildOrgName(w.getBuildOrgName());
            project.setUnitStripName(MhUnitStripEnum.getVal(w.getUnitStrip()));
            return project;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public List<ProjectLibListItemVO> options() {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode, Project::getProjectName)
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getCreateOn)
                .notIn(Project::getStatus, ProjectStatus.STOPPED_PASSED.getCode(),
                        ProjectStatus.ON_STOPPED_APPLY.getCode());
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        if (!userDetail.hasSuperAdmin()) {
            query.eq(Project::getBuildOrgCode, userDetail.getMhUnitIdStr());
        }
        List<Project> projects = projectService.list(query);
        return CollUtils.convert(projects, w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setId(w.getId());
            return item;
        });
    }

    public ProjectStoppedDetailVO detail(Long projectId) {
        Project project = projectService.getById(projectId);
        if (project == null) {
            throw ReturnException.wrap("项目不存在");
        }
        return ProjectStoppedDetailVO.builder()
                .projectId(project.getId())
                .projectName(project.getProjectName())
                .stoppedStatus(project.getStoppedStatus())
                .stoppedReason(project.getStoppedReason())
                .stoppedFiles(project.getStoppedFiles())
                .build();
    }

}
