package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.projectlib.entity.*;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.dto.QxProjectSystemReplaceInfosDTO;
import com.hz.pm.api.projectlib.model.enumeration.MhSystemReplaceType;
import com.hz.pm.api.projectlib.model.enumeration.QxProjectStatus;
import com.hz.pm.api.projectlib.model.req.*;
import com.hz.pm.api.projectlib.model.vo.QxProjectDetailVO;
import com.hz.pm.api.projectlib.model.vo.QxProjectSystemReplaceInfosVO;
import com.hz.pm.api.projectlib.model.vo.QxProjectVO;
import com.hz.pm.api.projectlib.model.vo.QxPurchaseVO;
import com.hz.pm.api.projectlib.service.*;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper;
import com.hz.pm.api.user.helper.MhUnitQueryAuthHelper.UnitQueryState;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * QxProjectManage
 * </p>
 *
 * @author WendyYang
 * @since 10:42 2024/8/31
 */
@Component
@RequiredArgsConstructor
public class QxProjectManage {

    private final MhUnitQueryAuthHelper unitQueryHelper;
    private final IQxProjectService qxProjectService;
    private final IMhSystemReplaceInfoService mhSystemReplaceInfoService;
    private final IQxProjectSystemReplaceInfosService qxProjectSystemReplaceInfosService;
    private final IQxProjectSystemReplaceInfosRelationService qxProjectSystemReplaceInfosRelationService;
    private final IQxPurchaseService qxPurchaseService;

    private void checkExistProject(String projectName, Long buildOrgCode) {
        LambdaQueryWrapper<QxProject> query = Wrappers.lambdaQuery(QxProject.class)
                .eq(QxProject::getProjectName, projectName)
                .eq(QxProject::getBuildOrgCode, buildOrgCode);
        if (qxProjectService.count(query) > 0) {
            throw ReturnException.wrap("项目已存在");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitProjectRecord(QxProjectRecordReq req) {
        checkExistProject(req.getProjectName(), req.getBuildOrgCode());
        QxProject project = new QxProject();
        project.setStatus(QxProjectStatus.DECLARED.getCode());
        BeanUtil.copyProperties(req, project);
        qxProjectService.save(project);
        if (CollUtil.isNotEmpty(req.getSourceSystemIds())) {
            List<String> sourceSystemIds = req.getSourceSystemIds();
            List<MhSystemReplaceInfo> systemReplaceInfos = mhSystemReplaceInfoService.listByIds(sourceSystemIds);
            Map<Boolean, List<MhSystemReplaceInfo>> replaceInfosMap = CollUtils.group(systemReplaceInfos,
                    w -> MhSystemReplaceType.IT.eq(w.getReplaceType()));
            List<QxProjectSystemReplaceInfos> projectSystemReplaceInfos = new ArrayList<>();
            List<MhSystemReplaceInfo> itReplaceInfos = replaceInfosMap.get(true);
            List<String> tmpSourceSystemIds = new ArrayList<>();
            if (CollUtil.isNotEmpty(itReplaceInfos)) {
                Map<String, List<MhSystemReplaceInfo>> group = CollUtils.group(itReplaceInfos, MhSystemReplaceInfo::getTargetName);
                for (Map.Entry<String, List<MhSystemReplaceInfo>> entry : group.entrySet()) {
                    List<MhSystemReplaceInfo> value = entry.getValue();
                    QxProjectSystemReplaceInfos item = new QxProjectSystemReplaceInfos();
                    item.setProjectId(project.getId());
                    item.setReplaceType(value.get(0).getReplaceType());
                    item.setTargetSystemName(value.get(0).getTargetName());
                    item.setSourceSystemNames(CollUtils.joinByComma(value, MhSystemReplaceInfo::getSourceName));
                    tmpSourceSystemIds.add(CollUtils.joinByComma(value, MhSystemReplaceInfo::getId));
                    projectSystemReplaceInfos.add(item);
                }
            }
            List<MhSystemReplaceInfo> otherReplaceInfos = replaceInfosMap.get(false);
            if (CollUtil.isNotEmpty(otherReplaceInfos)) {
                for (MhSystemReplaceInfo mhSystemReplaceInfo : otherReplaceInfos) {
                    QxProjectSystemReplaceInfos item = new QxProjectSystemReplaceInfos();
                    item.setProjectId(project.getId());
                    item.setReplaceType(mhSystemReplaceInfo.getReplaceType());
                    item.setTargetSystemName(mhSystemReplaceInfo.getTargetName());
                    item.setSourceSystemNames(mhSystemReplaceInfo.getSourceName());
                    tmpSourceSystemIds.add(mhSystemReplaceInfo.getId());
                    projectSystemReplaceInfos.add(item);
                }
            }
            if (CollUtil.isNotEmpty(projectSystemReplaceInfos)) {
                qxProjectSystemReplaceInfosService.saveBatch(projectSystemReplaceInfos);
                List<QxProjectSystemReplaceInfosRelation> relations = new ArrayList<>();
                for (int i = 0; i < tmpSourceSystemIds.size(); i++) {
                    for (String sourceSystemId : tmpSourceSystemIds.get(i).split(",")) {
                        QxProjectSystemReplaceInfosRelation relation = new QxProjectSystemReplaceInfosRelation();
                        relation.setProjectId(project.getId());
                        relation.setSourceSystemId(sourceSystemId);
                        relation.setReplaceInfosId(projectSystemReplaceInfos.get(i).getId());
                        relations.add(relation);
                    }
                }
                qxProjectSystemReplaceInfosRelationService.saveBatch(relations);
            }
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitPurchaseRecord(QxPurchaseRecordReq req) {
        QxProject project = qxProjectService.getById(req.getProjectId());
        if (!QxProjectStatus.DECLARED.eq(project.getStatus())) {
            throw ReturnException.wrap("当前状态不允许采购备案");
        }
        QxPurchase purchase = qxPurchaseService.getByProjectId(req.getProjectId());
        if (purchase == null) {
            purchase = new QxPurchase();
        }
        BeanUtil.copyProperties(req, purchase);
        qxPurchaseService.saveOrUpdate(purchase);
        updateQxProjectStatus(QxProjectStatus.PURCHASED, req.getProjectId());
    }

    private void updateQxProjectStatus(QxProjectStatus status, Long id) {
        qxProjectService.update(Wrappers.lambdaUpdate(QxProject.class)
                .set(QxProject::getUpdateOn, LocalDateTime.now())
                .set(QxProject::getStatus, status.getCode())
                .eq(QxProject::getId, id));
    }

    @Transactional(rollbackFor = Exception.class)
    public void submitAcceptRecord(QxAcceptRecordReq req) {
        QxProject project = qxProjectService.getById(req.getProjectId());
        if (!QxProjectStatus.UPGRADED.eq(project.getStatus())) {
            throw ReturnException.wrap("当前状态不允许验收备案");
        }
        Wrapper<QxPurchase> update = Wrappers.lambdaUpdate(QxPurchase.class)
                .eq(QxPurchase::getProjectId, req.getProjectId())
                .set(QxPurchase::getFinalAcceptDate, req.getFinalAcceptDate())
                .set(QxPurchase::getFinalAcceptFile, req.getFinalAcceptFile())
                .set(QxPurchase::getFirstAcceptDate, req.getFirstAcceptDate())
                .set(QxPurchase::getFirstAcceptFile, req.getFirstAcceptFile())
                .set(QxPurchase::getUpdateOn, LocalDateTime.now())
                .set(QxPurchase::getUpdateBy, LoginUserUtil.getUserId());
        qxPurchaseService.update(update);
        updateQxProjectStatus(QxProjectStatus.ACCEPTED, req.getProjectId());
    }

    @Transactional(rollbackFor = Exception.class)
    public void submitUpgradeRecord(QxUpgradeRecordReq req) {
        QxProject project = qxProjectService.getById(req.getProjectId());
        if (!QxProjectStatus.PURCHASED.eq(project.getStatus())) {
            throw ReturnException.wrap("当前状态不允许改造提升备案");
        }
        List<QxProjectSystemReplaceInfosDTO> replaceInfos = req.getSystemReplaceInfos();
        List<QxProjectSystemReplaceInfos> projectSystemReplaceInfos = new ArrayList<>();
        for (QxProjectSystemReplaceInfosDTO replaceInfo : replaceInfos) {
            QxProjectSystemReplaceInfos qxReplaceInfo = new QxProjectSystemReplaceInfos();
            BeanUtil.copyProperties(replaceInfo, qxReplaceInfo);
            projectSystemReplaceInfos.add(qxReplaceInfo);
        }
        qxProjectSystemReplaceInfosService.updateBatchById(projectSystemReplaceInfos);
        updateQxProjectStatus(QxProjectStatus.UPGRADED, req.getProjectId());
    }

    public QxProjectDetailVO getProjectDetail(Long projectId) {
        QxProject project = qxProjectService.getById(projectId);
        if (project == null) {
            return null;
        }
        QxProjectDetailVO projectDetail = new QxProjectDetailVO();
        QxProjectVO retProject = BeanUtil.copyProperties(project, QxProjectVO.class);
        projectDetail.setProject(retProject);
        QxPurchase purchase = qxPurchaseService.getByProjectId(projectId);
        if (purchase != null) {
            QxPurchaseVO retPurchase = BeanUtil.copyProperties(purchase, QxPurchaseVO.class);
            projectDetail.setPurchase(retPurchase);
        }
        List<QxProjectSystemReplaceInfos> systemReplaceInfos = qxProjectSystemReplaceInfosService.listByProjectId(projectId);
        if (CollUtil.isNotEmpty(systemReplaceInfos)) {
            List<QxProjectSystemReplaceInfosVO> retSystemReplaceInfos = BeanUtil.copyToList(systemReplaceInfos, QxProjectSystemReplaceInfosVO.class);
            projectDetail.setSystemReplaceInfos(retSystemReplaceInfos);
        }
        return projectDetail;
    }

    public PageVo<QxProjectVO> page(QxProjectLibReq req) {
        LambdaQueryWrapper<QxProject> query = Wrappers.lambdaQuery(QxProject.class)
                .eq(req.getProjectYear() != null, QxProject::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, QxProject::getUnitStrip, req.getUnitStrip())
                .eq(req.getStatus() != null, QxProject::getStatus, req.getStatus())
                .like(StrUtil.isNotBlank(req.getProjectName()), QxProject::getProjectName, req.getProjectName())
                .like(StrUtil.isNotBlank(req.getBuildOrgName()), QxProject::getBuildOrgName, req.getBuildOrgName())
                .exists(StrUtil.isNotBlank(req.getReplaceType()),
                        "select 1 from qx_project_system_replace_infos " +
                                "where qx_project_system_replace_infos.project_id = qx_project.id " +
                                "and replace_type = {0}", req.getReplaceType())
                .orderByDesc(QxProject::getUpdateOn);
        UnitQueryState unitQuery = unitQueryHelper.listCanViewUnitIds(req.getBuildOrgCode());
        if (!unitQuery.isState()) {
            return PageVo.empty();
        }
        BizUtils.notEmpty(unitQuery.getUnitIds(), w -> query.in(QxProject::getBuildOrgCode, w));
        Page<QxProject> page = qxProjectService.page(req.page(), query);
        return PageVo.of(BeanUtil.copyToList(page.getRecords(), QxProjectVO.class), page.getTotal());
    }

    public List<MhSystemReplaceInfoVO> listReplaceInfo(Long projectId) {
        Wrapper<QxProjectSystemReplaceInfosRelation> query = Wrappers.lambdaQuery(QxProjectSystemReplaceInfosRelation.class)
                .eq(QxProjectSystemReplaceInfosRelation::getProjectId, projectId);
        List<QxProjectSystemReplaceInfosRelation> relations = qxProjectSystemReplaceInfosRelationService.list(query);
        if (CollUtil.isEmpty(relations)) {
            return Collections.emptyList();
        }
        List<String> systemIds = CollUtils.fieldList(relations, QxProjectSystemReplaceInfosRelation::getSourceSystemId);
        List<MhSystemReplaceInfo> replaceInfos = mhSystemReplaceInfoService.listByIds(systemIds);
        return BeanUtil.copyToList(replaceInfos, MhSystemReplaceInfoVO.class);
    }

}
