package com.hz.pm.api.scheduler.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.StopWatch;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectRenewalFundDeclaration;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.ProjectRenewalApprovalStatusEnum;
import com.hz.pm.api.projectlib.model.enumeration.WarningFlowTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.WarningOperationTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.service.IProjectRenewalFundDeclarationService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.enumeration.ProjectEarlyWarningStatusEnum;
import com.hz.pm.api.sys.model.entity.ProjectEarlyWarning;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import com.hz.pm.api.sys.service.IProjectEarlyWarningService;
import com.wflow.enums.WarningRuleTypeEnum;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.nullness.qual.Nullable;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author ZPF
 * @date 2023/8/3 上午9:53
 * 预警填报 超时任务
 */
@Component
@Slf4j
@RequiredArgsConstructor
@ConditionalOnExpression("${early-warning-without-submit.open:true}")
public class EarlyWarningProjectTask {

    private final IProjectService projectService;
    private final IProjectEarlyWarningService projectEarlyWarningService;
    private final IEarlyWarningRecordsService earlyWarningRecordsService;
    private final TaskService taskService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    @Scheduled(fixedDelay = 5, initialDelay = 5, timeUnit = TimeUnit.MINUTES)
    public void doEarlyWarningDelayed() {
        log.info("=========== 预警项目状态校验任务开始 ===========");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE));

        if (CollUtil.isEmpty(projects)) {
            return;
        }

        Set<String> projectSet = Sets.newHashSet();
        Map<String, Project> projectMap = projects.stream().filter(p -> projectSet.add(p.getProjectCode()))
                .collect(Collectors.toMap(Project::getProjectCode, p -> p));
        List<ProjectEarlyWarning> warnings = projectEarlyWarningService.list();

        // 删除 已经不存在的项目
        List<ProjectEarlyWarning> toRemove = warnings.stream()
                .filter(w -> !projectMap.containsKey(w.getProjectCode()))
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(toRemove)) {
            List<Long> removeIds = toRemove.stream().map(ProjectEarlyWarning::getId)
                    .collect(Collectors.toList());
            projectEarlyWarningService.removeBatchByIds(removeIds);
        }

        //  去查询 各个报警 是否已经正常
        List<ProjectEarlyWarning> trueWarnings = warnings.stream().filter(w -> Objects.nonNull(w.getNormal()) && !w.getNormal()).collect(Collectors.toList());
        if (CollUtil.isEmpty(trueWarnings)) {
            return;
        }
        checkNormal(trueWarnings);
        stopWatch.stop();
        log.info("=========== 预警项目状态校验任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

    //@Scheduled(fixedDelay = 5, initialDelay = 5, timeUnit = TimeUnit.MINUTES)
    public void doEarlyWarningDeclared() {
        log.info("=========== 预警项目状态校验任务开始 ===========");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE));

        if (CollUtil.isEmpty(projects)) {
            return;
        }

        Set<String> projectSet = Sets.newHashSet();
        Map<String, Project> projectMap = projects.stream().filter(p -> projectSet.add(p.getProjectCode()))
                .collect(Collectors.toMap(Project::getProjectCode, p -> p));
        List<ProjectEarlyWarning> warnings = projectEarlyWarningService.list();

        // 删除 已经不存在的项目
        List<ProjectEarlyWarning> toRemove = warnings.stream()
                .filter(w -> !projectMap.containsKey(w.getProjectCode()))
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(toRemove)) {
            List<Long> removeIds = toRemove.stream().map(ProjectEarlyWarning::getId)
                    .collect(Collectors.toList());
            projectEarlyWarningService.removeBatchByIds(removeIds);
        }

        //  去查询 各个报警 是否已经正常
        List<ProjectEarlyWarning> trueWarnings = warnings.stream().filter(w -> Objects.nonNull(w.getNormal()) && !w.getNormal()).collect(Collectors.toList());
        if (CollUtil.isEmpty(trueWarnings)) {
            return;
        }
        checkNormal(trueWarnings);
        stopWatch.stop();
        log.info("=========== 预警项目状态校验任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

    // 3.去查询 各个报警 是否已经正常
    private void checkNormal(List<ProjectEarlyWarning> trueWarnings) {
        // 滞后预警 如果正常了 要改回正常
        List<ProjectEarlyWarning> delayWarning = trueWarnings.stream().filter(w ->
                        Objects.nonNull(w.getDelayWarning())
                                && w.getDelayWarning())
                .collect(Collectors.toList());
        checkDelayWarning(delayWarning);
    }

    private void checkDelayWarning(List<ProjectEarlyWarning> delayWarning) {
        if (CollUtil.isEmpty(delayWarning)) {
            return;
        }
        for (ProjectEarlyWarning delay : delayWarning) {
            String projectCode = delay.getProjectCode();
            Project project = projectService.getProjectByCode(projectCode);
            if (Objects.isNull(project)) {
                continue;
            }
            WflowEarlyWarningRecords record = earlyWarningRecordsService.getOne(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .eq(WflowEarlyWarningRecords::getProjectCode, projectCode)
                    .eq(WflowEarlyWarningRecords::getRuleType, WarningRuleTypeEnum.DELAY_WARNING.getCode())
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime)
                    .last(BizConst.LIMIT_1));
            if (Objects.isNull(record)) {
                continue;
            }
            Integer biz = record.getBiz();
            WarningFlowTypeEnum flowTypeEnum = WarningFlowTypeEnum.getByCode(biz);
            if (Objects.isNull(flowTypeEnum)) {
                continue;
            }
            ProjectStatus projectStatus = flowTypeEnum.getProjectStatus();
            // 已经不是这个状态了 说明已经被提交了
            if (!projectStatus.eq(project.getStatus())) {
                checkCommited(delay, flowTypeEnum, projectCode);
                // 如果滞后预警正常了 改回正常
                if (Boolean.FALSE.equals(delay.getDelayWarning())){
                    delay.setNormal(Boolean.TRUE);
                    delay.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                }
                projectEarlyWarningService.updateById(delay);
            }
        }
    }

    private void checkDeclaredWarning(List<ProjectEarlyWarning> declaredWarnings) {
        if (CollUtil.isEmpty(declaredWarnings)) {
            return;
        }
        for (ProjectEarlyWarning declaredWarning : declaredWarnings) {
            String projectCode = declaredWarning.getProjectCode();
            Project project = projectService.getProjectByCode(projectCode);
            if (Objects.isNull(project)) {
                continue;
            }
            WflowEarlyWarningRecords record = earlyWarningRecordsService.getOne(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .eq(WflowEarlyWarningRecords::getProjectCode, projectCode)
                    .eq(WflowEarlyWarningRecords::getRuleType, WarningRuleTypeEnum.DECLARED_WARNING.getCode())
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime)
                    .last(BizConst.LIMIT_1));
            if (Objects.isNull(record)) {
                continue;
            }
            Integer biz = record.getBiz();
            WarningFlowTypeEnum flowTypeEnum = WarningFlowTypeEnum.getByCode(biz);
            if (Objects.isNull(flowTypeEnum)) {
                continue;
            }
            ProjectStatus projectStatus = flowTypeEnum.getProjectStatus();
            // 已经不是这个状态了 根据选择的节点判断是否完成了提交
            if (!projectStatus.eq(project.getStatus())) {
                checkCommited(declaredWarning, flowTypeEnum, projectCode);
                // 其它几种 都没有 说明已经正常了
                if (Boolean.FALSE.equals(declaredWarning.getProcessWarning()) &&
                        Boolean.FALSE.equals(declaredWarning.getDelayWarning())) {
                    declaredWarning.setNormal(Boolean.TRUE);
                    declaredWarning.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                }
                projectEarlyWarningService.updateById(declaredWarning);
            }
        }
    }

    private void checkCommited(ProjectEarlyWarning warning, WarningFlowTypeEnum flowTypeEnum, String projectCode) {
        switch (flowTypeEnum){
            // 已立项未采购 当用户采购&合同备案完成 说明进度正常
            case APPROVED_PROJECT_NOT_PURCHASED: {
                List<ProjectStateChangeEvent> events = Lists.newArrayList();
                events.add(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                // 说明进行了采购&合同信息备案
                if (Objects.nonNull(projectStatusChange)) {
                    if (WarningRuleTypeEnum.DECLARED_WARNING.getCode().equals(warning.getRuleType())) {
                        warning.setDeclaredWarning(Boolean.FALSE);
                    }else if (WarningRuleTypeEnum.DELAY_WARNING.getCode().equals(warning.getRuleType())){
                        warning.setDelayWarning(Boolean.FALSE);
                    }
                }
            }
            break;
            // 已采购未改造完成 当用户适配改造完成已完成单位确认 说明进度正常
            case PURCHASED_UNMODIFIED:{
                List<TenderStateChangeEvent> events = Lists.newArrayList();
                events.add(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                PurchaseStatusChange purchaseStatusChange = purchaseStatusChangeService.getLastOne(projectCode, events);
                // 说明进行了单位确认
                if (Objects.nonNull(purchaseStatusChange)) {
                    if (WarningRuleTypeEnum.DECLARED_WARNING.getCode().equals(warning.getRuleType())) {
                        warning.setDeclaredWarning(Boolean.FALSE);
                    }else if (WarningRuleTypeEnum.DELAY_WARNING.getCode().equals(warning.getRuleType())){
                        warning.setDelayWarning(Boolean.FALSE);
                    }
                }

            }
            break;
            // 已改造完成未终验 当用户提交终验申请 说明进度正常
            case MODIFIED_NOT_FINAL_INSPECTION:{
                List<ProjectStateChangeEvent> events = Lists.newArrayList();
                events.add(ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION);
                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                // 说明提交了终验申请
                if (Objects.nonNull(projectStatusChange)) {
                    if (WarningRuleTypeEnum.DECLARED_WARNING.getCode().equals(warning.getRuleType())) {
                        warning.setDeclaredWarning(Boolean.FALSE);
                    }else if (WarningRuleTypeEnum.DELAY_WARNING.getCode().equals(warning.getRuleType())){
                        warning.setDelayWarning(Boolean.FALSE);
                    }
                }
            }
            break;
        }
    }

    /**
     * 流程 查看 是否已经处理过
     *
     * @param processWarnings
     */
    private void checkProcessWarning(List<ProjectEarlyWarning> processWarnings) {
        if (CollUtil.isEmpty(processWarnings)) {
            return;
        }
        for (ProjectEarlyWarning processWarning : processWarnings) {
            String projectCode = processWarning.getProjectCode();
            WflowEarlyWarningRecords record = earlyWarningRecordsService.getOne(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .eq(WflowEarlyWarningRecords::getProjectCode, projectCode)
                    .orderByDesc(WflowEarlyWarningRecords::getWarningTime)
                    .last(BizConst.LIMIT_1));
            if (Objects.isNull(record)) {
                continue;
            }
            String nodeId = record.getNodeId();
            if (StringUtils.isBlank(nodeId)) {
                continue;
            }
            List<Task> tasks = taskService.createTaskQuery()
                    .taskDefinitionKey(nodeId)
                    .list();
            if (CollUtil.isEmpty(tasks)) {
                //说明 已经审批通过了
                processWarning.setProcessWarning(Boolean.FALSE);
                //其它三种 都没有 说明已经正常了
                if (Boolean.FALSE.equals(processWarning.getDeclaredWarning()) &&
                        Boolean.FALSE.equals(processWarning.getDelayWarning())) {
                    processWarning.setNormal(Boolean.TRUE);
                    processWarning.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                }
                projectEarlyWarningService.updateById(processWarning);
            }
        }

    }
}
