package com.hz.pm.api.scheduler.task;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.model.entity.ProjectEarlyWarning;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.util.EnvironmentUtil;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.WarningFlowTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.WarningNoticeTypeEnum;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.scheduler.contants.TaskConstant;
import com.hz.pm.api.sys.enumeration.ProjectEarlyWarningStatusEnum;
import com.hz.pm.api.sys.manage.EarlyWarningManage;
import com.hz.pm.api.sys.service.IProjectEarlyWarningService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.entity.UserInfo;
import com.hz.pm.api.user.model.enumeration.RoleEnum;
import com.hz.pm.api.user.service.IUserInfoService;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.enums.WarningRuleTypeEnum;
import com.wflow.service.IEarlyWarningService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.StopWatch;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * @author ZPF
 * @date 2023/8/3 上午9:53
 * 预警填报 超时任务
 */
@Slf4j
@Component
@RequiredArgsConstructor
@ConditionalOnExpression("${early-warning-without-submit.open:true}")
public class EarlyWarningWithoutSubmitTask {

    private final IEarlyWarningService earlyWarningService;
    private final IProjectService projectService;
    private final EarlyWarningManage earlyWarningManage;
    private final IUserInfoService userInfoService;
    private final IProjectEarlyWarningService projectEarlyWarningService;
    private final MhUnitCache mhUnitCache;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final EnvironmentUtil environmentUtil;
    private final UserInfoHelper userInfoHelper;
    private final IEarlyWarningRecordsService earlyWarningRecordsService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    private ChronoUnit getTimeUnit() {
        if (environmentUtil.isDevEnv()) {
            return ChronoUnit.MINUTES;
        } else {
            return ChronoUnit.DAYS;
        }
    }

    /**
     * 滞后预警 每两分钟执行一次
     */
    @Scheduled(cron = "${early-warning-without-submit-delay-warning.cron:0 0/2 * * * ?}")
    public void doEarlyWarningDelayed() {
        log.info("=========== 滞后预警任务开始 ===========");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        // 获取项目库所有建设单位不为空的项目
        LambdaQueryWrapper<Project> queryWrapper = Wrappers.lambdaQuery(Project.class)
                .isNotNull(Project::getBuildOrgCode)
                .eq(Project::getNewest, Boolean.TRUE);
        List<Project> projects = projectService.list(queryWrapper);
        List<ProjectEarlyWarning> projectEarlyWarnings = Lists.newArrayList();
        List<WflowEarlyWarningRecords> earlyWarningRecords = Lists.newArrayList();
        List<ProjectEarlyWarning> needRemoved = Lists.newArrayList();
        // 查询当前预警表中 已存在的的滞后预警数据
        Wrapper<ProjectEarlyWarning> warningQuery = Wrappers.lambdaQuery(ProjectEarlyWarning.class)
                .eq(ProjectEarlyWarning::getRuleType, WarningRuleTypeEnum.DELAY_WARNING.getCode());
        List<ProjectEarlyWarning> earlyWarnings = projectEarlyWarningService.list(warningQuery);
        Map<String, ProjectEarlyWarning> projectEarlyWarningMap = CollUtils.listToMap(earlyWarnings, c -> c.getProjectCode() + StrPool.DASH + c.getStepName());
        // 遍历项目 获取对应申报单位的滞后预警规则
        for (Project project : projects) {
            String buildOrgCode = project.getBuildOrgCode();
            String projectCode = project.getProjectCode();
            Long mhUnitId = Long.valueOf(buildOrgCode);
            List<Long> unitIdPaths = mhUnitCache.getUnitIdPaths(mhUnitId);
            if (unitIdPaths.isEmpty()) {
                log.error("该单位无法进行滞后预警：{}", mhUnitId);
                continue;
            }
            List<String> unitIdPathsStr = CollUtils.convert(unitIdPaths, String::valueOf);
            String orderSql = String.format(" order by field(mh_unit_id, %s) desc limit 1",
                    CollUtils.join(unitIdPaths, w -> "'" + w + "'", StrUtil.COMMA));
            // 从当前建设单位开始向上获取滞后预警规则
            WflowEarlyWarning warning = earlyWarningService.getOne(Wrappers.lambdaQuery(WflowEarlyWarning.class)
                    .eq(WflowEarlyWarning::getRuleType, WarningRuleTypeEnum.DELAY_WARNING.getCode())
                    .eq(WflowEarlyWarning::getIsOpen, true)
                    .in(WflowEarlyWarning::getMhUnitId, unitIdPathsStr)
                    .last(orderSql));
            if (Objects.isNull(warning)){
                log.info("未找到对应单位：{} 的滞后预警规则", mhUnitId);
                continue;
            }
            String rule = warning.getRule();
            if (StringUtils.isNotBlank(rule)) {
                JSONArray ruleArray = JSON.parseArray(rule);
                // 对JSONArray 根据"biz"字段从小到大排序
                ruleArray.sort((o1, o2) -> {
                    JSONObject o1Json = JSON.parseObject(JSON.toJSONString(o1));
                    JSONObject o2Json = JSON.parseObject(JSON.toJSONString(o2));
                    Integer o1Biz = o1Json.getInteger("biz");
                    Integer o2Biz = o2Json.getInteger("biz");
                    return o1Biz.compareTo(o2Biz);
                });
                if (CollUtil.isNotEmpty(ruleArray)) {
                    ruleArray.forEach(r -> {
                        JSONObject rJson = JSON.parseObject(JSON.toJSONString(r));
                        Integer biz = rJson.getInteger("biz");
                        Integer delayTime = rJson.getInteger("delayTime");
                        String notice = rJson.getString("notice");
                        WarningFlowTypeEnum flowType = WarningFlowTypeEnum.getByCode(biz);
                        if (Objects.isNull(delayTime) || Objects.isNull(biz) || flowType == null) {
                            log.warn("规则数据 错误 :{}", rJson);
                            return;
                        }
                        String path = flowType.getPath();
                        Map<String, ProjectStatusChange> projectStatusChangeMap = MapUtil.newHashMap();
                        String stepName = null;
                        List<String> projectCodes = Lists.newArrayList();
                        projectCodes.add(project.getProjectCode());
                        // 是否进行采购&合同备案
                        boolean isPurchase = false;
                        // 适配改造是否完成单位确认
                        boolean isModified = false;
                        // 终验申请是否提交
                        boolean isFinalInspection = false;
                        switch (flowType) {
                            // 已立项未采购 获取 立项时间
                            case APPROVED_PROJECT_NOT_PURCHASED:{
                                stepName = TaskConstant.StepName.PURCHASE;
                                if (CollUtil.isNotEmpty(projectCodes)) {
                                    Map<String, ProjectStatusChange> approvedProjectNotPurchasedMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                            ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                                    List<ProjectStateChangeEvent> events = Lists.newArrayList();
                                    events.add(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                                    ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                                    // 说明进行了采购&合同信息备案
                                    if (Objects.nonNull(projectStatusChange)) {
                                        isPurchase = true;
                                    }
                                    if (CollUtil.isNotEmpty(approvedProjectNotPurchasedMap)) {
                                        projectStatusChangeMap.putAll(approvedProjectNotPurchasedMap);
                                    }
                                }
                            }
                            break;
                            // 已采购未改造完成 获取合同签订时间
                            case PURCHASED_UNMODIFIED:{
                                stepName = TaskConstant.StepName.ADAPT_MODIFY;
                                if (CollUtil.isNotEmpty(projectCodes)) {
                                    Map<String, ProjectStatusChange> purchasedUnmodifiedMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                            ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                                    if (CollUtil.isNotEmpty(purchasedUnmodifiedMap)) {
                                        projectStatusChangeMap.putAll(purchasedUnmodifiedMap);
                                    }
                                    List<TenderStateChangeEvent> events = Lists.newArrayList();
                                    events.add(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                                    PurchaseStatusChange purchaseStatusChange = purchaseStatusChangeService.getLastOne(projectCode, events);
                                    // 说明进行了单位确认
                                    if (Objects.nonNull(purchaseStatusChange)) {
                                        isModified = true;
                                    }
                                }
                            }
                            break;
                            // 已改造完成未终验 获取单位确认时间
                            case MODIFIED_NOT_FINAL_INSPECTION:{
                                stepName = TaskConstant.StepName.FINAL;
                                if (CollUtil.isNotEmpty(projectCodes)) {
                                    Map<String, ProjectStatusChange> modifiedNotFinalInspectionMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                            ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                                    if (CollUtil.isNotEmpty(modifiedNotFinalInspectionMap)) {
                                        projectStatusChangeMap.putAll(modifiedNotFinalInspectionMap);
                                    }
                                }
                                List<ProjectStateChangeEvent> events = Lists.newArrayList();
                                events.add(ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION);
                                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectCode, events);
                                // 说明提交了终验申请
                                if (Objects.nonNull(projectStatusChange)) {
                                    isFinalInspection = true;
                                }
                            }
                            break;
                            default:
                                log.error("Unexpected value: {}", flowType);
                        }
                        // 需要首页监测预警展示的项目
                        // 获取项目 滞后(历时)时间
                        HashMap<String, LocalDateTime> touchTimeMap = MapUtil.newHashMap();
                        String finalStepName = stepName;
                        ProjectStatusChange change = projectStatusChangeMap.get(projectCode);
                        LocalDateTime touchTime = change == null ? null : change.getCreateOn();
                        // 是否进行预警
                        boolean isTouch = isTouch(flowType, isPurchase, isModified, isFinalInspection);
                        if (touchTime != null) {
                            touchTimeMap.put(projectCode + StrPool.DASH + biz, touchTime);
                            long duration = getDurationTime(touchTime,LocalDateTime.now(),getTimeUnit());
                            // 如果对应节点超期 且未提交至下一步
                            if (duration > delayTime && isTouch) {
                                List<ProjectEarlyWarning> projectEarlyWarningList = Lists.newArrayList();
                                List<WflowEarlyWarningRecords> earlyWarningRecordsList = Lists.newArrayList();
                                // 项目 滞后预警
                                projectEarlyWarningList = projectEarlyWarningService.earlyWarning(project, WarningRuleTypeEnum.DELAY_WARNING.getCode(), com.wflow.enums.WarningNoticeTypeEnum.DELAY.getCode(), finalStepName, projectEarlyWarningList);
                                projectEarlyWarnings.addAll(projectEarlyWarningList);

                                // 目前没有通知
                                // 如果去预警通知人 1.超级管理员 2.单位管理员 如果都包含 都要发
                                List<String> batchEmployees = Lists.newArrayList();
                                if (StringUtils.isNotBlank(notice) && notice.contains(WarningNoticeTypeEnum.SUPER_ADMIN.getCode().toString())) {
                                    // 获取超级管理员用户信息
                                    List<UserInfo> superAdminUsers = userInfoHelper.listUsersByRole(RoleEnum.SUPER_ADMIN);
                                    List<String> superAdminEmployeeCodes = superAdminUsers.stream().map(UserInfo::getEmployeeCode).collect(Collectors.toList());
                                    batchEmployees.addAll(superAdminEmployeeCodes);
                                }
                                if (StringUtils.isNotBlank(notice) && notice.contains(WarningNoticeTypeEnum.UNIT_ADMIN.getCode().toString())) {
                                    // 获取单位管理员用户信息
                                    List<UserInfo> unitAdminUsers = userInfoHelper.listUsersByRole(RoleEnum.COMPANY_MANAGER);
                                    // 过滤出当前项目所属单位的单位管理员
                                    List<String> unitAdminEmployeeCodes = unitAdminUsers.stream().filter(user -> Objects.equals(user.getMhUnitId().toString(), project.getBuildOrgCode()))
                                            .map(UserInfo::getEmployeeCode).collect(Collectors.toList());
                                    batchEmployees.addAll(unitAdminEmployeeCodes);
                                }
                                // 保存到预警记录中
                                earlyWarningRecordsList = earlyWarningManage.getEarlyWarningDelayRecords(warning.getNoticeMethod(), biz, project,
                                        WarningRuleTypeEnum.DELAY_WARNING.getCode(), ProjectEarlyWarningStatusEnum.DELAY_TIME.getCode(),
                                        path, touchTimeMap, earlyWarningRecordsList);
                                earlyWarningRecords.addAll(earlyWarningRecordsList);
                            }else {
                                // 判断是否已经存在对应节点预警 如果有 从预警信息表中移除
                                if (projectEarlyWarningMap.containsKey(project.getProjectCode() + StrPool.DASH + finalStepName)) {
                                    ProjectEarlyWarning projectEarlyWarning = projectEarlyWarningMap.get(projectCode + StrPool.DASH + finalStepName);
                                    needRemoved.add(projectEarlyWarning);
                                }
                            }
                        }
                    });
                }
            }
        }
        // 每个规则按通知节点顺序 更新保存一次
        if (CollUtil.isNotEmpty(needRemoved)){
            projectEarlyWarningService.removeBatchByIds(needRemoved);
        }
        if (CollUtil.isNotEmpty(projectEarlyWarnings)){
            log.info("需要滞后预警通知的项目数：{}", projectEarlyWarnings.size());
            projectEarlyWarningService.saveOrUpdateBatch(projectEarlyWarnings);
        }
        if (CollUtil.isNotEmpty(earlyWarningRecords)){
            log.info("需要滞后预警通知记录数：{}", earlyWarningRecords.size());
            earlyWarningRecordsService.saveOrUpdateBatch(earlyWarningRecords);
        }
        stopWatch.stop();
        log.info("=========== 滞后预警任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

    private boolean isTouch(WarningFlowTypeEnum flowType, Boolean isPurchase, Boolean isModified, Boolean isFinalInspection) {
        switch (flowType) {
            case APPROVED_PROJECT_NOT_PURCHASED:
                return !isPurchase;
            case PURCHASED_UNMODIFIED:
                return !isModified;
            case MODIFIED_NOT_FINAL_INSPECTION:
                return !isFinalInspection;
            default:
                return false;
        }
    }

    private long getDurationTime(LocalDateTime touchTime, LocalDateTime now, ChronoUnit timeUnit) {
        return timeUnit.between(touchTime, now);
    }

    /**
     * 填报预警
     */
    //@Scheduled(cron = "${early-warning-without-submit.cron:0 0/5 * * * ?}")
    public void doEarlyWarningDeclared() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        // 1.查询 填报的 预警规则 填报类型的 每个区域 每个规则
        Wrapper<WflowEarlyWarning> query = Wrappers.lambdaQuery(WflowEarlyWarning.class)
                .eq(WflowEarlyWarning::getRuleType, WarningRuleTypeEnum.DECLARED_WARNING.getCode())
                .eq(WflowEarlyWarning::getIsOpen, true);
        List<WflowEarlyWarning> warnings = earlyWarningService.list(query);
        for (WflowEarlyWarning warning : warnings) {
            long mhUnitId = warning.getMhUnitId();
            List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(mhUnitId);
            String noticeContent = warning.getNoticeContent();
            String adventContent = warning.getAdventContent();
            String rule = warning.getRule();
            if (StringUtils.isNotBlank(rule)) {
                JSONArray ruleArray = JSON.parseArray(rule);
                if (CollUtil.isNotEmpty(ruleArray)) {
                    ruleArray.forEach(r -> {
                        JSONObject rJson = JSON.parseObject(JSON.toJSONString(r));
                        Integer time = rJson.getInteger("time");
                        Integer biz = rJson.getInteger("biz");
                        String notice = rJson.getString("notice");
                        Integer adventTime = rJson.getInteger("adventTime");
                        WarningFlowTypeEnum flowType = WarningFlowTypeEnum.getByCode(biz);
                        if (((Objects.isNull(time) && Objects.isNull(adventTime)) || Objects.isNull(biz))
                                || StrUtil.isBlank(notice)
                                || flowType == null) {
                            log.warn("规则数据 错误 :{}", rJson);
                            return;
                        }
                        String path = flowType.getPath();
                        //查询 所有这个单位 未提交的项目
                        Wrapper<Project> projectQuery = Wrappers.lambdaQuery(Project.class)
                                .eq(Project::getBuildOrgCode, mhUnitId)
                                .eq(Project::getNewest, Boolean.TRUE)
                                .eq(Project::getStatus, flowType.getProjectStatus().getCode());
                        List<Project> needNextProjects = projectService.list(projectQuery);
                        Map<String, List<Integer>> noticeMap = Maps.newHashMap();
                        List<String> projectCodes = CollUtils.fieldList(needNextProjects, Project::getProjectCode);
                        Map<String, ProjectStatusChange> projectStatusChangeMap;
                        switch (flowType) {
                            // 已立项未采购
                            case APPROVED_PROJECT_NOT_PURCHASED:{
                                projectStatusChangeMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                        ProjectStateChangeEvent.SUBMIT_PURCHASE_NOTICE);
                            }
                            break;
                            // 已采购未改造完成
                            case PURCHASED_UNMODIFIED:{
                                projectStatusChangeMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                        ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                            }
                            break;
                            // 已改造完成未终验
                            case MODIFIED_NOT_FINAL_INSPECTION:{
                                projectStatusChangeMap = projectStatusChangeService.listLastEventMap(projectCodes,
                                        ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                            }
                            break;
                            default:
                                return;
                        }
                        // 需要发通知的项目
                        List<Project> needToWaringProjects = needNextProjects.stream()
                                .filter(p -> {
                                    ProjectStatusChange change = projectStatusChangeMap.get(p.getProjectCode());
                                    List<Integer> noticeTypes = Lists.newArrayList();
                                    LocalDateTime touchTime = change == null ? null : change.getCreateOn();
                                    if (touchTime != null) {
                                        if (Objects.nonNull(time)) {
                                            long duration = Duration.between(touchTime, LocalDateTime.now()).get(getTimeUnit());
                                            if (duration >= time && duration < time + (environmentUtil.isDevEnv() ? 5 : 1)) {
                                                noticeTypes.add(com.wflow.enums.WarningNoticeTypeEnum.OVER.getCode());
                                            }
                                        }
                                        if (Objects.nonNull(adventTime)) {
                                            long duration = Duration.between(touchTime, LocalDateTime.now()).get(getTimeUnit());
                                            if (duration >= adventTime && duration < adventTime + (environmentUtil.isDevEnv() ? 5 : 1)) {
                                                noticeTypes.add(com.wflow.enums.WarningNoticeTypeEnum.ADVENT.getCode());
                                            }
                                        }
                                        if (CollUtil.isNotEmpty(noticeTypes)) {
                                            noticeMap.put(p.getProjectCode(), noticeTypes);
                                            //项目 填报预警
                                            projectEarlyWarningService.earlyWarning(p, WarningRuleTypeEnum.DECLARED_WARNING.getCode(), noticeTypes);
                                            return Boolean.TRUE;
                                        }
                                    }
                                    return Boolean.FALSE;
                                }).collect(Collectors.toList());

                        for (Project needToWaringProject : needToWaringProjects) {
                            String username;
                            String mobile;
                            //去预警通知人 1.超级管理员 2.单位管理员 如果都包含 都要发
                            List<String> batchEmployees = Lists.newArrayList();
                            if (notice.contains(WarningNoticeTypeEnum.SUPER_ADMIN.getCode().toString())) {
                                String employeeCode = null;
                                username = needToWaringProject.getContactName();
                                mobile = needToWaringProject.getContactPhone();
                                if (StrUtils.isAllNotBlank(username, mobile)) {
                                    UserInfo user = userInfoService.getByUserNameAndMobile(username, mobile);
                                    if (Objects.nonNull(user)) {
                                        employeeCode = user.getEmployeeCode();
                                    }
                                    if (Objects.nonNull(employeeCode)) {
                                        batchEmployees.add(employeeCode);
                                    }
                                }
                            }
                            if (notice.contains(WarningNoticeTypeEnum.UNIT_ADMIN.getCode().toString())) {
                                String employeeCode = null;
                                username = needToWaringProject.getResponsibleMan();
                                mobile = needToWaringProject.getResponsibleManMobile();
                                if (StrUtils.isAllNotBlank(username, mobile)) {
                                    UserInfo user = userInfoService.getByUserNameAndMobile(username, mobile);
                                    if (Objects.nonNull(user)) {
                                        employeeCode = user.getEmployeeCode();
                                    }
                                    if (Objects.nonNull(employeeCode)) {
                                        batchEmployees.add(employeeCode);
                                    }
                                }
                            }
                            if (CollUtil.isNotEmpty(batchEmployees)) {
                                String batchEmployeesStr = String.join(StrPool.COMMA, batchEmployees);
                                if (noticeMap.containsKey(needToWaringProject.getProjectCode())) {
                                    List<Integer> noticeTypes = noticeMap.get(needToWaringProject.getProjectCode());
                                    for (Integer noticeType : noticeTypes) {
                                        for (String employeeCode : batchEmployees) {
                                            earlyWarningManage.doEarlyWarning(warning.getNoticeMethod(), Objects.nonNull(noticeType) && noticeType.equals(1) ? noticeContent : adventContent,
                                                    adventTime, time, biz,
                                                    needToWaringProject.getUpdateOn(), Long.valueOf(employeeCode), needToWaringProject,
                                                    WarningRuleTypeEnum.DECLARED_WARNING.getCode(), noticeType, path, batchEmployeesStr,
                                                    null);
                                        }
                                    }
                                }
                            }
                        }
                    });
                }
            }
        }

        stopWatch.stop();
        log.info("=========== 预警填报超时任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

}
