package com.hz.pm.api.todocenter.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.projectdeclared.model.entity.Xinchuang;
import com.hz.pm.api.projectdeclared.service.IXinchuangService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.todocenter.handle.PassHandle;
import com.hz.pm.api.todocenter.handle.WithDrawHandle;
import com.hz.pm.api.todocenter.model.req.ProcessDetailReq;
import com.hz.pm.api.todocenter.model.vo.ProcessProgressDetailVO;
import com.hz.pm.api.todocenter.service.ITodoService;
import com.hz.pm.api.todocenter.util.BuildUserUtils;
import com.hz.pm.api.todocenter.util.ProcessUtil;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.process.enums.NodeTypeEnum;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.HistoryService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * TodoServiceImpl
 * </p>
 *
 * @author WendyYang
 * @since 14:53 2024/4/16
 */
@Service
@AllArgsConstructor
@Slf4j
public class TodoServiceImpl implements ITodoService {

    private final ProcessInstanceService processInstanceService;
    private final BuildUserUtils buildUserUtils;
    private final UserInfoHelper userInfoHelper;
    private final WithDrawHandle withDrawHandle;
    private final PassHandle passHandle;
    private final IProjectService projectService;
    private final IProjectInstService projectInstService;

    private final IXinchuangService xinchuangService;

    private final HistoryService historyService;

    /**
     * 查询流程表单数据及审批的进度步骤
     *
     * @param request 请求参数
     * @return 流程进度及表单详情
     */
    @Override
    public ProcessProgressDetailVO getProcessDetail(ProcessDetailReq request) {
        String employeeCode = LoginUserUtil.loginUserDetail().getUserIdStr();
        String instanceId = request.getInstanceId();
        String nodeId = request.getNodeId();
        Long projectId = request.getProjectId();
        String taskId = request.getTaskId();
        ProcessProgressVo instDetail = processInstanceService.getProgressInstanceDetail(nodeId, instanceId, taskId);

        if (Objects.isNull(instDetail)) {
            return null;
        }

        List<ProgressNode> progressInfo = instDetail.getProgressInfo();
        Boolean isHighLine = Boolean.FALSE;
        if (CollUtil.isNotEmpty(progressInfo)) {
            buildUserUtils.buildUserByProcessInfo(progressInfo);
            isHighLine = checkHighLine(progressInfo, taskId, LoginUserUtil.loginUserDetail());
        }
        ProcessProgressDetailVO res = new ProcessProgressDetailVO();
        res.setProcessProgressVo(instDetail);
        res.setStatus(instDetail.getStatus());
        res.setProjectId(projectId);
        res.setCanWithdraw(withDrawHandle.checkCanWithdraw(instanceId, instDetail, request.getTaskId()));
        res.setIsHighLine(isHighLine);
        passHandle.checkCanPassOrSeal(request.getInstanceId(), request.getTaskId(), employeeCode, res);
        //是不是被 驳回|退回
        res.setIsChange(isChangeRecord(projectId));
        return res;
    }

    /**
     * 筛选 当前登录人 是不是 上级条线单位的审批
     *
     * @param progressInfo \
     * @param taskId       \
     * @param user         \
     * @return \
     */
    private boolean checkHighLine(List<ProgressNode> progressInfo, String taskId, UserInfoDetails user) {
        if (CollUtil.isEmpty(progressInfo)) {
            return Boolean.FALSE;
        }

        boolean state = Boolean.FALSE;
        for (ProgressNode p : progressInfo) {
            //如果是 子流程
            if (Objects.nonNull(p.getNodeType())
                    && NodeTypeEnum.SUB.name().equals(p.getNodeType().name())
                    && (Boolean.TRUE.equals(p.getIsHighLine()))) {
                List<ProgressNode> children = p.getChildren();
                ProgressNode progressNode = children.get(0);
                String userId = progressNode.getUserId();
                if (StringUtils.isNotBlank(userId)) {
                    UserFullInfoDTO currUser = userInfoHelper.getUserFullInfo(userId);
                    if (Objects.nonNull(currUser)
                            && user.getMhUnitId().equals(currUser.getMhUnitId())
                            && (ProcessUtil.containsTask(children, taskId))) {
                        state = Boolean.TRUE;
                        break;
                    }
                }
            }
        }
        return state;
    }

    /**
     * 判断 当前项目是不是被驳回 或者 退回过
     *
     * @param projectId \
     * @return \
     */
    @Override
    public boolean isChangeRecord(Long projectId) {
        //1.先判断下 项目存不存在
        Project project = projectService.getById(projectId);
        if (Objects.isNull(project)) {
            return Boolean.FALSE;
        }

        List<Long> projectIds = projectService.allVersionProjectIds(project.getProjectCode());
        //2.要判断 项目在当前状态 有没有被驳回和退回过
        //当前项目状态流程的 实例 看看是不是有2个以上 有2个说明 有退回 驳回
        List<ProjectInst> pis = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIds)
                .orderByDesc(ProjectInst::getInstType)
                .last("limit 2"));
        if (CollUtil.isEmpty(pis)) {
            return Boolean.FALSE;
        }
        //判断 项目关联的最新的2个实例 如果类型不匹配 也不属于是变更了
        //还要看 项目当前的流程实例code 是不是 关联的最新实例
        if (pis.size() < 2 ||
                !pis.get(0).getInstCode().equals(project.getInstCode()) ||
                pis.get(0).getProjectId().equals(pis.get(1).getProjectId()) ||
                !pis.get(0).getInstType().equals(pis.get(1).getInstType())) {
            return Boolean.FALSE;
        }

        return Boolean.TRUE;
    }

    @Override
    public boolean isChangeRecordXinchuang(Long xinchuangId) {
        //1.先判断下 信创存不存在
        Xinchuang xinchuang = xinchuangService.getById(xinchuangId);
        if (Objects.isNull(xinchuang) || StringUtils.isBlank(xinchuang.getInstCode())) {
            return Boolean.FALSE;
        }

        String instCode = xinchuang.getInstCode();

        HistoricProcessInstance instance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(instCode)
                .singleResult();
        if(StringUtils.isNotBlank(instance.getEndActivityId()) &&
                "refuse-end".equals(instance.getEndActivityId())){
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }
}
